/********************************************************************
 FileName:     	HardwareProfile - SimbuckSpeedBox.h
 Dependencies:  See INCLUDES section
 Processor:     PIC18 USB Microcontrollers
 Hardware:      Simbuck Baseboard4550
 Compiler:      Microchip C18
 Company:       Deben Enterprises Limited


 THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
 File Description:

 Change History:
  Rev   Date         Description
  1.0   03/03/2012   Initial release

********************************************************************/

#ifndef HARDWARE_PROFILE_SIMBUCKSPEEDBOX_H
#define HARDWARE_PROFILE_SIMBUCKSPEEDBOX_H


    /*******************************************************************/
    /******** USB stack hardware selection options *********************/
    /*******************************************************************/
    //This section is the set of definitions required by the MCHPFSUSB
    //  framework.  These definitions tell the firmware what mode it is
    //  running in, and where it can find the results to some information
    //  that the stack needs.
    //These definitions are required by every application developed with
    //  this revision of the MCHPFSUSB framework.  Please review each
    //  option carefully and determine which options are desired/required
    //  for your application.

    //The PICDEM FS USB Demo Board platform supports the USE_SELF_POWER_SENSE_IO
    //and USE_USB_BUS_SENSE_IO features.  Uncomment the below line(s) if
    //it is desireable to use one or both of the features.
   //#define USE_SELF_POWER_SENSE_IO
    #define tris_self_power     TRISAbits.TRISA0    // Input
    #if defined(USE_SELF_POWER_SENSE_IO)
    #define self_power          PORTAbits.RA0
    #else
    #define self_power          1
    #endif

   // #define USE_USB_BUS_SENSE_IO
    #define tris_usb_bus_sense  TRISAbits.TRISA1    // Input
    #if defined(USE_USB_BUS_SENSE_IO)
    #define USB_BUS_SENSE       PORTAbits.RA1
    #else
    #define USB_BUS_SENSE       1
    #endif


    //Uncomment the following line to make the output HEX of this
    //  project work with the MCHPUSB Bootloader
    //#define PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER

    //Uncomment the following line to make the output HEX of this
    //  project work with the HID Bootloader
    #define PROGRAMMABLE_WITH_USB_HID_BOOTLOADER

    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/
    /******** Application specific definitions *************************/
    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/

/** L E D s *******************************************************************/

    #define mInitAllLEDs()      LATE &= 0xF0; TRISE &= 0xF0; 

    #define mLED_1              LATEbits.LATE0
    #define mLED_2              LATEbits.LATE1
    #define mLED_3              LATEbits.LATE2

    #define mGetLED_1()         mLED_1
    #define mGetLED_2()         mLED_2
    #define mGetLED_3()         mLED_3

    #define mLED_1_On()         mLED_1 = 1;
    #define mLED_2_On()         mLED_2 = 1;
    #define mLED_3_On()         mLED_3 = 1;

    #define mLED_1_Off()        mLED_1 = 0;
    #define mLED_2_Off()        mLED_2 = 0;
    #define mLED_3_Off()        mLED_3 = 0;

    #define mLED_1_Toggle()     mLED_1 = !mLED_1;
    #define mLED_2_Toggle()     mLED_2 = !mLED_2;
    #define mLED_3_Toggle()     mLED_3 = !mLED_3;


/** S W I T C H  D A T A **********************************************/

/* Switch inputs  ******************************************************
*	TRISDbits.TRISD3 = INPUT_PIN		// input 3  0xnn
*	TRISDbits.TRISD2 = INPUT_PIN		// input 2  0xnn
*	TRISDbits.TRISD1 = INPUT_PIN		// input 1  0xnn
*	TRISDbits.TRISD0 = INPUT_PIN		// input 0  0xnn
***********************************************************************/

#define mInitRIP()		TRISD |= 0x0F;

#define mRIP_3			LATDbits.LATD3;
#define mRIP_2			LATDbits.LATD2;
#define mRIP_1			LATDbits.LATD1;
#define mRIP_0			LATDbits.LATD0;

/** O U T P U T  D A T A **********************************************/

/* Debug Alarm outputs  ******************************************************
*	TRISDbits.TRISD7 = OUTPUT_PIN		// Debug alarm 0  OPTO_4  MOLEX Pin 11
*	TRISDbits.TRISD6 = OUTPUT_PIN		// Debug alarm 1  OPTO_3  MOLEX Pin 9
*	TRISDbits.TRISD5 = OUTPUT_PIN		// Debug alarm 2  OPTO_2  MOLEX Pin 7
*	TRISDbits.TRISD4 = OUTPUT_PIN		// Debug alarm 3  OPTO_1  MOLEX Pin 5
***********************************************************************/

#define mInitROP()		TRISEbits.TRISE0=OUTPUT_PIN;TRISEbits.TRISE1=OUTPUT_PIN;TRISEbits.TRISE2=OUTPUT_PIN;LATEbits.LATE2=INPUT_PIN;

#define mROP_1_2		LATEbits.LATE0
#define mROP_1_2_Open() 	mROP_1_2 = 1;
#define mROP_1_2_Closed()	mROP_1_2 = 0;
#define mROP_1_2_Toggle()	mROP_1_2 = !ROP_1_2;

#define mROP_3_4		LATEbits.LATE1
#define mROP_3_4_Open()		mROP_3_4 = 1;
#define mROP_3_4_Closed() 	mROP_3_4 = 0;
#define mROP_3_4_Toggle()       mROP_3_4 = !ROP_3_4;

#define mROP_5  		LATEbits.LATE2
#define mROP_5_Open()		mROP_5 = 1;
#define mROP_5_Closed() 	mROP_5 = 0;
#define mROP_5_Toggle()         mROP_5 = !mROP_5;


/** S I G N A L S ***********************************************************
*	TRISDbits.TRISD7 = OUTPUT_PIN;		// CPS
*	TRISDbits.TRISD6 = OUTPUT_PIN;		// CID
*	TRISDbits.TRISD5 = OUTPUT_PIN;		// CID inverted
*	TRISDbits.TRISD4 = OUTPUT_PIN;		// REV/MIN
*****************************************************************************/
#define mInit_OPTO()		TRISDbits.TRISD7=OUTPUT_PIN;LATDbits.LATD7=0;TRISDbits.TRISD6=OUTPUT_PIN;LATDbits.LATD6=0;TRISDbits.TRISD5=OUTPUT_PIN;LATDbits.LATD5=0;TRISDbits.TRISD4=OUTPUT_PIN;LATDbits.LATD4=0;

#define mCPS			LATDbits.LATD7
#define mCPS_Hi() 	   	mCPS = 1;
#define mCPS_Lo()		mCPS = 0;
#define mCPS_Toggle()  	   	mCPS = !mCPS;

#define mCID			LATDbits.LATD6
#define mCID_Hi() 	   	mCID = 1;
#define mCID_Lo()		mCID = 0;
#define mCID_Toggle()  	   	mCID = !mCID;

#define mCID_inv		LATDbits.LATD5
#define mCID_inv_Hi() 	   	mCID_inv = 1;
#define mCID_inv_Lo()		mCID_inv = 0;
#define mCID_inv_Toggle()  	MCID_inv = !mCID_inv;

#define mREV			LATDbits.LATD4
#define mREV_Hi() 	   	mREV = 1;
#define mREV_Lo()		mREV = 0;
#define mREV_Toggle()  	   	mREV = !mREV;

/** S I G N A L S ***********************************************************
*	TRISBbits.TRISB2 = OUTPUT_PIN;		// EDGE GENERATOR
*****************************************************************************/

#define mEDGE			LATBbits.LATB2
#define mEDGE_Hi() 	   	mEDGE = 1;
#define mEDGE_Lo()		mEDGE = 0;
#define mEDGE_Toggle()  	mEDGE = !mEDGE;




/** D E B U G ****************************************************************
*	TRISBbits.TRISB3 = OUTPUT_PIN;		// 'Scope pin for de-bugging
*****************************************************************************/

#define mALERT			LATBbits.LATB3
#define mALERT_Hi() 	   	mALERT = 1;
#define mALERT_Lo()		mALERT = 0;
#define mALERT_Toggle()  	mALERT = !mALERT;



/** I/O pin definitions ********************************************/
#define INPUT_PIN           1
#define OUTPUT_PIN          0
#define SELECT              0
#define DESELECT            1
 

#endif  //HARDWARE_PROFILE_SIMBUCKSPEEDBOX_H
