/*******************************************************************************
 *
 *                Simbuck Firmware Version 1.0
 *
 *******************************************************************************
 * FileName:        simbuck_cfg.h
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 3.42
 * Company:         Deben Enterprises Ltd.
 *******************************************************************************
 File Description:

 Change History:
  Rev   Description
  ----  -----------------------------------------
  1.0  First release

*******************************************************************************/
#ifndef SIMBUCK_CFG_H
#define SIMBUCK_CFG_H

/** D E V I C E S *************************************************************/
//#define MY_4550_DEVICE
//#define MY_4685_DEVICE

/** B A S E B O A R D S *******************************************************/
/* Uncomment the baseboard that you are using */
//#define SIMBUCK_BASEBOARD_4550
#define SIMBUCK_BASEBOARD_4685

/** B O O T L O A D E R *******************************************************/

/* Uncomment the Bootloader that you are using */
//#define HID_BOOTLOADER              // Microchip modified HID Bootloader
//#define TINY_BOOTLOADER           // not implemented
//#define PUF_BOOTLOADER            // not implemented
//#define CAN_BOOTLOADER            // to be implemented
#define SERIAL_BOOTLOADER         // Microchip modified Serial Bootloader


/** S I M B L O C K S *********************************************************/
/* Uncomment the hardware simblocks that you are using */
//#define SIMBLOCK_DUAL_CAN
#define SIMBLOCK_ADC

// SELF TESTS
//#define USE_DUAL_CAN_SELFTEST
//#define USE_ADC_SELF_TEST

/** S I M B L O C K S   C O N T R O L *****************************************/

  #if defined(SIMBLOCK_DUAL_CAN)
    #define USE_SPI                 // set up the MCP2515 CAN SPI pins
    #define USE_CAN_1               // This must be active as it runs the clock(s)
    #define USE_CAN_2               // You can turn this off if only one node required
  #endif

/** H A R D W A R E   D I S P L A Y *******************************************/
/* Uncomment the hardware display that you are using */
#define USE_STATUS_LED



/** P E R I P H E R A L S   C O N T R O L *************************************/
 #if defined(SIMBUCK_BASEBOARD_4550)
//#define USE_SELF_POWER_SENSE_IO
//#define USE_USB_BUS_SENSE_IO
#endif


#if defined(SIMBUCK_BASEBOARD_4685)
 //#define USE_ECAN
 #define USE_EUSART
#endif

//#define USE_SPI
//#define USE_EUSART
//#define USE_PWM
//#define USE_ADC


/** B A S E   F I R M W A R E   V E R S I O N *********************************/
#if defined(SIMBUCK_BASEBOARD_4550)
  #define DEBUG_VERSION_BASE    0x00
  #define MINOR_VERSION_BASE    0x02
  #define MAJOR_VERSION_BASE    0x01

/** Base Firmware content ******************************************************

version 0.0.1
=============   ================================================================
Bootloading
EEPROM read and write
Firmware Versioning

Basic Interrupt handling (Low and High ISR)
TMR0 - 1ms interval
SPI connectivity
CAN Controller configuration

================================================================================

version 0.1.1
================================================================================
 Implemented HID Bootloader for Windows 7 capability

================================================================================

 version 0.1.2
================================================================================
  Added 'watch' files for Simbuck-JPF

================================================================================

*******************************************************************************/


/** A P P L I C A T I O N   F I R M W A R E   V E R S I O N *******************/
// If a device is built using this file then update the device version here

 #if defined(MY_4550_DEVICE)
  #define DEBUG_VERSION_DEVICE    0x00
  #define MINOR_VERSION_DEVICE    0x03
  #define MAJOR_VERSION_DEVICE    0x00
 #endif
#endif

/** Application Firmware content ***********************************************

 version 0.1.0
 -------------   ---------------------------------------------------------------
 simbuckDualCANSelfTest enabled
 -------------------------------------------------------------------------------

 version 0.2.0
 -------------   ---------------------------------------------------------------
 create example device
 Use Timer and Potentiometer via ADC for Edge Generation
--------------------------------------------------------------------------------

version 0.3.0
 -------------   ---------------------------------------------------------------
 create example device
 Capability for retrieving CAN2515 Address value

*******************************************************************************/


#if defined(SIMBUCK_BASEBOARD_4685)
  #define DEBUG_VERSION_BASE    0x00
  #define MINOR_VERSION_BASE    0x01
  #define MAJOR_VERSION_BASE    0x01


/** Base Firmware content ******************************************************

version 0.0.1
================================================================================
 Bootloading
 EEPROM read and write
 Firmware Versioning

 Basic Interrupt handling (Low and High ISR)
 TMR0 - 1ms interval

================================================================================

 version 0.1.0
================================================================================
  Added 'watch' files for Simbuck-JPF


 version 0.1.1
================================================================================
 Implemented Serial Bootloader AN1310 v1.05r

================================================================================


 -------------------------------------------------------------------------------

/** A P P L I C A T I O N   F I R M W A R E   V E R S I O N *******************/
// If a device is built using this file then update the device version here

 #if defined(MY_4685_DEVICE)
  #define DEBUG_VERSION_DEVICE    0x01
  #define MINOR_VERSION_DEVICE    0x00
  #define MAJOR_VERSION_DEVICE    0x00
 #endif
#endif

/** Application Firmware content ***********************************************

 version 0.0.0
 -------------   ---------------------------------------------------------------

 -------------------------------------------------------------------------------

*******************************************************************************/



/**  3 r d  P A R T Y  L I B R A R Y  V E R S I O N S *************************/

#if defined(SIMBUCK_BASEBOARD_4550)
  #if defined(MCHPFSUSB)
    /* PICDEM FS USB Demo Version */
    #define USB_STACK_LIB       0x01    //  MCHPUSB Usb stack
    #define MINOR_VERSION_USB   0x02    //  Version 1.2
    #define MAJOR_VERSION_USB   0x01
    #define BOOTLOAD_TYPE       0x01

  #elif defined(HID_BOOTLOADER)
    /* Microchip Solutions Library v2011-07-16 */
    #define USB_STACK_LIB       0x02    // HID USB stack
    #define STACK_YEAR_USB      0x0B	// 2011
    #define STACK_MONTH_USB     0x07	// 07
    #define STACK_DAY_USB       0x0C	// 12
    //need these until Java app is fixed - used in Simbuck Signal Generator
    #define MINOR_VERSION_USB   0x02    //   Version 1.2
    #define MAJOR_VERSION_USB   0x01
    #define BOOTLOAD_TYPE       0x02

    //#define USB_MAJOR_VER     0x02    provided in usb.h
    //#define USB_MINOR_VER     0x06    provided in usb.h
    //#define USB_DOT_VER       0x01    provided in usb.h

   #define SIMBUCK_BOOT_MAJOR   0x01   // First release - based off USB stack 0x02
   #define SIMBUCK_BOOT_MINOR   0x01
   #define SIMBUCK_BOOT_DOT     0x00

   #else
     #define BOOTLOAD_TYPE      0x00
   #endif
#endif

#if defined(SIMBUCK_BASEBOARD_4685)
   #if defined(CAN_BOOTLOADER)
     #define BOOTLOAD_TYPE      0x06
   #elif defined(SERIAL_BOOTLOADER)

    #define MINOR_VERSION_SERIAL   0x05    //   AN1310 Version 1.05
    #define MAJOR_VERSION_SERIAL   0x01
    #define DOT_VERSION_SERIAL     0x00

     #define BOOTLOAD_TYPE      0x07

     #define SIMBUCK_BOOT_MAJOR   0x01   // First release - based off
     #define SIMBUCK_BOOT_MINOR   0x01   // Serial Bootloader AN1310 v1.05
     #define SIMBUCK_BOOT_DOT     0x00

   #else
     #define BOOTLOAD_TYPE      0x00
   #endif
#endif

#if defined(SIMBUCK_BASEBOARD_PIC32)
    /* Microchip Solutions Library v2011-07-16 */
    #define USB_STACK_LIB       0x02    // HID USB stack
    #define STACK_YEAR_USB      0x0B	// 2011
    #define STACK_MONTH_USB     0x07	// 07
    #define STACK_DAY_USB       0x0C	// 14
    //need these until Java app is fixed - used in Simbuck Signal Generator
    #define MINOR_VERSION_USB   0x02    //   Version 1.2
    #define MAJOR_VERSION_USB   0x01
    #define BOOTLOAD_TYPE       0x01

    //#define USB_MAJOR_VER     0x02    provided in usb.h
    //#define USB_MINOR_VER     0x06    provided in usb.h
    //#define USB_DOT_VER       0x01    provided in usb.h

#endif

#endif //SIMBUCK_CFG_H
