/********************************************************************
 FileName:		CAN2515drv.c
 Dependencies:	        See INCLUDES section
 Processor:		PIC18F4550
 Hardware:		This demo is intended to be used on Simbuck Baseboard4550
                        boards supported by the MCHPFSUSB stack.
 Complier:  	        Microchip C18 (for PIC18)
 Company:		Deben Enterprises Ltd



 THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

 ********************************************************************
 File Description:

 Change History:
  Rev   Description
  ----  -----------------------------------------
  1.0  First release

 ********************************************************************

/** I N C L U D E S **********************************************************/
#include "simbuck_cfg.h"
#include "../HardwareProfile.h"
#include "../HardwareProfile - SimblockDualCAN.h"
#include "GenericTypeDefs.h"


#if defined(SIMBUCK_BASEBOARD_4550) 
#include <p18cxxx.h>
#include "../system/spi/simbuckspi.h"
#include "CAN2515drv.h"
#endif

#if defined(SIMBUCK_BASEBOARD_PIC32)
#include <plib.h>
#include "../system/spi/simbuckspi.h"
#include "CAN2515drv.h"
#endif


/** V A R I A B L E S ********************************************************/
#pragma udata
CAN_FLAGS CANFlags;

unsigned char CAN_1_rcv[14];
unsigned char CAN_2_rcv[14];
unsigned char CAN_3_rcv[14];
unsigned char CAN_4_rcv[14];

unsigned char CAN_update = 0x00; // 0000 0000 no updates
// 0000 0001 rcv 1 update
// 0000 0010 rcv 3 update
// 0000 0100 rcv 2 update
// 0000 1000 rcv 4 update

/** D E C L A R A T I O N S **************************************************/
#pragma code

/******************************************************************************/
/* 2515 Level '1' SPI functions							*/
/* Reference document - Microchip Datasheet 210801e				*/
/* Document Title - Stand-Alone CAN Controller with SPI Interface		*/
/* Document Chapter - Section 12 - SPI Interface				*/
/* Implement: 									*/
/*  RESET									*/
/*  READ									*/
/*  Read RX Buffer								*/
/*  WRITE									*/
/*  Load TX Buffer								*/
/*  RTS (Request To Send)							*/
/*  Read Status									*/
/*  RX Status								      */
/*  Bit Modify								      */
/*									      */
/******************************************************************************/


/******************************************************************************/

/* RESET
 */

void CAN2515_reset(void) {
    unsigned char ticCnt;
    unsigned char retVal;

    retVal = WriteSPI(CAN2515_RESET);

    // need to wait 128 tics
    for (ticCnt = 0; ticCnt < 130; ticCnt++);

}

/*********************************************************************************/

/* READ
 */

char CAN2515_read(unsigned char addr) {
    unsigned char ReturnValue;

    CAN2515_select();

    while (WriteSPI(CAN2515_READ));
    while (WriteSPI(addr));

    ReturnValue = ReadSPI(SPI_CAN_DUMMY);

    CAN2515_unselect();

    return (ReturnValue);
}

char CAN2515_readRx(unsigned char addr, unsigned char type) {
    unsigned char ReturnValue;
    unsigned char i;

    CAN2515_select();

    while (WriteSPI(CAN2515_READ));
    while (WriteSPI(addr));

    if (!CANFlags.cs_CAN_1) {

        for (i = 0; i < 13; i++) {
            if (type == 0) {
                CAN_1_rcv[i] = ReadSPI(SPI_CAN_DUMMY);
                CAN_update |= 0x01;
            } else {
                CAN_3_rcv[i] = ReadSPI(SPI_CAN_DUMMY);
                CAN_update |= 0x02;
            }
        }
    }

    if (!CANFlags.cs_CAN_2 == 1) {

        for (i = 0; i <= 13; i++) {
            if (type == 0) {
                CAN_2_rcv[i] = ReadSPI(SPI_CAN_DUMMY);
                CAN_update |= 0x04;
            } else {
                CAN_4_rcv[i] = ReadSPI(SPI_CAN_DUMMY);
                CAN_update |= 0x08;
            }
        }

    }

    CAN2515_unselect();

    return (ReturnValue);
}

/*********************************************************************************/

/* Read RX Buffer
 */

unsigned char CAN2515_read_rxbuffer(unsigned char addr) {

    unsigned char ReturnValue;

    CAN2515_select();

    while (WriteSPI(CAN2515_READ_RX_BUFF & addr));

    ReturnValue = ReadSPI(SPI_CAN_DUMMY);

    CAN2515_unselect();

    return ReturnValue;
}

/*********************************************************************************/

/* WRITE
 */

void CAN2515_write(unsigned char addr, unsigned char byte) {

    CAN2515_select();

    while (WriteSPI(CAN2515_WRITE));
    while (WriteSPI(addr));
    while (WriteSPI(byte));

    CAN2515_unselect();

}

/*********************************************************************************/

/* Load TX Buffer
 */

unsigned char CAN2515_loadtx(unsigned char buff) {
    unsigned char retVal;

    CAN2515_select();

    retVal = WriteSPI(0x40 | 0x31);

    CAN2515_unselect();

    return retVal;

}

/*********************************************************************************/

/* RTS (RequestToSend)
 */

unsigned char CAN2515_rts(unsigned char buff) {
    unsigned char retVal;

    CAN2515_select();

    retVal = WriteSPI(buff);

    CAN2515_unselect();

    return retVal;

}


/*********************************************************************************/

/* Read Status
 */

unsigned char CAN2515_readstatus(void) {
    unsigned char status;

    CAN2515_select();

    WriteSPI(CAN2515_READ_STATUS); // Send code for Read Status command
    status = ReadSPI(SPI_CAN_DUMMY);

    CAN2515_unselect();

    return status;

}


/*********************************************************************************/

/* RX Status
 */

unsigned char CAN2515_read_rxstatus(void) {
    unsigned char status;

    CAN2515_select();

    while (WriteSPI(CAN2515_RX_STATUS)); // Send code for Read RX status command
    status = ReadSPI(SPI_CAN_DUMMY);

    CAN2515_unselect();

    return status;

}


/*********************************************************************************/

/* Bit Modify
 */

unsigned char CAN2515_bitmodify(unsigned char addr, unsigned char mask, unsigned char value) {
    CAN2515_select();

    while (WriteSPI(CAN2515_BIT_MODIFY)); // Send code for bit modify command
    while (WriteSPI(addr)); // Send address of register to modify
    while (WriteSPI(mask)); // Send mask of bits to modify in the register
    while (WriteSPI(value)); // Send new data to be loaded into specified bits

    CAN2515_unselect();

    return (0);

}



/******************************************************************************/

/* Select the correct Controller Chip Select Pin
/*
/*   cs_mcp2515_CAN_x is defined in HardwareProfile - SimblockDualCAN
/*   
 */


char CAN2515_select(void) {


    char ERR = OK;

    if (!CANFlags.cs_CAN_1) {
        cs_mcp2515_CAN_1 = 0; // Select the hardware
        ERR = 1;
    } else if (!CANFlags.cs_CAN_2) {
        cs_mcp2515_CAN_2 = 0; // Select the hardware
        ERR = 2;
    } else {
        // we should set an error for no CS
        ERR = -1;
        return ERR;
    }
}


/********************************************************************************/

/* De-select the Controller Chip Select Pin
 */

void CAN2515_unselect() {
    // cs_mcp2515_CAN_x is defined in HardwareProfile - SimblockDualCAN

    cs_mcp2515_CAN_1 = 1;
    cs_mcp2515_CAN_2 = 1;
}



