/********************************************************************
 FileName:		CAN_MsgList.c
 Dependencies:	See INCLUDES section
 Processor:		PIC18 and PIC32 USB Microcontrollers
 Hardware:		This demo is intended to be used on Simbuck Baseboard4550
 				boards supported by the MCHPFSUSB stack.
 Complier:  	Microchip C18 (for PIC18), C32 (for PIC32)
 Company:		Deben Enterprises Ltd



 THIS SOFTWARE IS PROVIDED IN AN ?AS IS? CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
 File Description:

 Change History:
  Rev   Description
  ----  -----------------------------------------
  1.0  First release

********************************************************************


/** I N C L U D E S **********************************************************/
#include "simbuck_cfg.h"
#include "../HardwareProfile.h"

#if defined(SIMBUCK_BASEBOARD_4550)
 #include <p18cxxx.h>
#endif

#if defined(SIMBUCK_BASEBOARD_PIC32)
 #include <plib.h>
#endif

#include "../HardwareProfile - SimblockDualCAN.h"
#include "GenericTypeDefs.h"
#include "CAN_MsgList.h"
#include "CAN2515drv.h"
#include "CAN2515utils.h"
#include "CAN2515user.h"

/** V A R I A B L E S ********************************************************/
#pragma udata
CAN_MESSAGES CAN_messages;
extern CAN_FLAGS CANFlags;
extern CAN_PERF CANPerf;

/** D E C L A R A T I O N S *************************************************/
#pragma code

/*****************************************************************************/
/* 2515 high level functions									   */
/* Reference document - Microchip Datasheet 210801e				   */
/* Document Title - Stand-Alone CAN Controller with SPI Interface		   */
/* 														   */
/*	Defines the message transmit buffers for the user level CAN layer: 	   */
/*		CAN_startDiagSession								   */
/*		CAN_flowcontrol									   */
/*														   */
/*    Helper functions:										   */
/*		CAN_clearIntFlags									   */
/*														   */
/*****************************************************************************/


/*****************************************************************************/
/* CAN messages -> tx buffer 									   */
/*  send at a high rate to overwite the original messages @10ms
*/



/********************************************************************************/
/* CAN messages -> tx buffer 							*/
/*										*/

void myCAN1diagmessage(void)
{
	//CANFlags.cs_CAN_1 = 1;

//	Nop();
	setMsgTarget(CAN2515_REG_TXB0CTRL, BATT_PACK_FRONT, _STANDARD);
//	Nop();

	CAN2515_write(TXB0D0, PORTB);
	CAN2515_write(TXB0D1, CANPerf.msgs_sent_can_1);
	CAN2515_write(TXB0D2, CANPerf.msgs_rcvd_can_2);
	CAN2515_write(TXB0D3, 0x00);
	CAN2515_write(TXB0D4, 0x00);
	CAN2515_write(TXB0D5, 0x00);
	CAN2515_write(TXB0D6, 0x11);
	CAN2515_write(TXB0D7, 0x11);
	
	//CANFlags.cs_CAN_1 = 0;

}


void myCAN2diagmessage(void)
{
	//CANFlags.cs_CAN_2 = 1;

	//Nop();
	setMsgTarget(CAN2515_REG_TXB0CTRL, BATT_PACK_REAR, _STANDARD);
	//Nop();

	CAN2515_write(TXB0D0, PORTB);
	CAN2515_write(TXB0D1, CANPerf.msgs_sent_can_2);
	CAN2515_write(TXB0D2, CANPerf.msgs_rcvd_can_2);
	CAN2515_write(TXB0D3, 0x00);
	CAN2515_write(TXB0D4, 0x00);
	CAN2515_write(TXB0D5, 0x00);
	CAN2515_write(TXB0D6, 0x22);
	CAN2515_write(TXB0D7, 0x22);
	
	//CANFlags.cs_CAN_2 = 0;

}

void myCANdiagmessage(void)
{
	//CANFlags.cs_CAN_2 = 1;

	//Nop();
	setMsgTarget(CAN2515_REG_TXB1CTRL, BATT_MON_DIAG, _STANDARD);
        //Nop();
        //setMsgTarget(CAN2515_REG_TXB1CTRL, BATT_MON_DIAG, _STANDARD);
	//Nop();

	CAN2515_write(TXB1D0, 0x00);
	CAN2515_write(TXB1D1, 0x00);
	CAN2515_write(TXB1D2, 0x00);
	CAN2515_write(TXB1D3, 0x00);
	CAN2515_write(TXB1D4, 0x00);
	CAN2515_write(TXB1D5, 0x00);
	CAN2515_write(TXB1D6, 0x55);
	CAN2515_write(TXB1D7, 0x55);

	//CANFlags.cs_CAN_2 = 0;

}

void createResponseList(void)
{

  CAN_messages.BATT_PACK_FRONT_MSG  = (encodeRespondToID(BATT_PACK_FRONT, _STANDARD)) >> 16;
  CAN_messages.BATT_PACK_REAR_MSG  = (encodeRespondToID(BATT_PACK_REAR, _STANDARD)) >> 16;
  CAN_messages.BATT_MON_DIAG_MSG  = (encodeRespondToID(BATT_MON_DIAG, _STANDARD)) >> 16;
}

