/********************************************************************
 FileName:		CANProcessMsgHS.c
 Dependencies:	See INCLUDES section
 Processor:		PIC18 and PIC32 USB Microcontrollers
 Hardware:		This demo is intended to be used on Simbuck Baseboard4550
 				boards supported by the MCHPFSUSB stack.
 Complier:  	Microchip C18 (for PIC18), C32 (for PIC32)
 Company:		Deben Enterprises Ltd



 THIS SOFTWARE IS PROVIDED IN AN ?AS IS? CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
 File Description:

 Change History:
  Rev   Description
  ----  -----------------------------------------
  1.0  First release

********************************************************************


/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include "../HardwareProfile - Baseboard4550.h"
#include "simbuck_cfg.h"

#if defined(SIMBLOCK_DUAL_CAN)
#include "GenericTypeDefs.h"
#include "system/CAN/CAN_useHS.h"
#include "system/CAN/CAN2515drv.h"
#include "system/CAN/CAN_MsgList.h"
#include "system/CAN/CAN_ProcessMsgHS.h"
#include "user/user.h"

/** V A R I A B L E S ********************************************************/
#pragma udata
extern CAN_FLAGS CANFlags;
extern CAN_MESSAGES CAN_messages;

extern CAN_FRAME canFrame_CAN_1;
extern CAN_FRAME canFrame_ls;
extern CAN_FRAME canFrame_CAN_1_diag;
extern CAN_FRAME canFrame_ls_diag;

//extern CAN_FRAME canFrame;

extern unsigned char *canFrame_CAN_1_ptr;
extern unsigned char *canFrame_CAN_1_diag_ptr;

LRG_MSG largeMessage;
LRG_MSG *largeMessage_ptr;


/** P R O T T Y P E S ********************************************************/

/** D E C L A R A T I O N S *************************************************/
#pragma code

/****************************************************************************/
/* 2515 high level functions												*/
/****************************************************************************/

void process_CAN_1CAN_diag_msg(void)
{
	unsigned char testVal;
	int i = 0;
	int myTemp;

	// received an expected diagnostic response
	CAN_RecvDiagResponse();
	
	//V70_data.diagmsg_cnt++;	
	//CANFlags.msgs_proc --;

		 Nop();
		 if(canFrame_CAN_1_diag.D0 == 0x10)
			 {
			  largeMessage.rcvBytes = canFrame_CAN_1_diag.D1;
			  largeMessage.numBytes = 0;
			  for(i=0; i<7; i++)
  		        {
			     
 				 largeMessage.msg_20[i] = canFrame_CAN_1_diag_ptr[i+4];
  		        }
	
			  CANFlags.cs_CAN_1CAN = 1;
	
			   CAN_flowcontrol();
			   CANsend(_B0);
 			   
			CAN_waitForDiagResponse();		 		

				CANFlags.lrgMsgToFrame = 0x20;	
			    largeMessage.numBytes = largeMessage.numBytes + 7;

		  	// Added this AFTER working code
			   canFrame_CAN_1_diag.IDENTIFIER = 0x0000;
			   
			 } 
			
			if((canFrame_CAN_1_diag.D0 >= 0x21) &&
			   (canFrame_CAN_1_diag.D0 <= 0x29))
				{
				 // capture the Byte(s) of interest  	
				 Nop();
				 largeMessage.ID = canFrame_CAN_1_diag.IDENTIFIER;
				
				 switch(canFrame_CAN_1_diag.D0)
					{
					// 21 - 29 are largeMessage return bytes
					  
					 case 33:
					  for(i=0; i<7; i++)
		  		        {
						 myTemp = canFrame_CAN_1_diag_ptr[i];	 
						 largeMessage.msg_21[i] =  canFrame_CAN_1_diag_ptr[i+4];
  	      		        }
						CANFlags.lrgMsgToFrame = 0x21;	
	  				    largeMessage.numBytes = largeMessage.numBytes + 7;

					 break;

					 case 34:
					  for(i=0; i<7; i++)
		  		        {
				         largeMessage.msg_22[i] =  canFrame_CAN_1_diag_ptr[i+4];
  	      		        }	
						CANFlags.lrgMsgToFrame = 0x52;
					    largeMessage.numBytes = largeMessage.numBytes + 7;

					 break;

					 case 35:
					  for(i=0; i<7; i++)
		  		        {
				         largeMessage.msg_23[i] =  canFrame_CAN_1_diag_ptr[i+4];
  	      		        }	
						CANFlags.lrgMsgToFrame = 0x53;
				        largeMessage.numBytes = largeMessage.numBytes + 7;

					 break;

					 case 36:
					  for(i=0; i<7; i++)
		  		        {
				         largeMessage.msg_24[i] =  canFrame_CAN_1_diag_ptr[i+4];
  	      		        }	
						CANFlags.lrgMsgToFrame = 0x54;
						largeMessage.numBytes = largeMessage.numBytes + 7;

					 break;

					 case 37:
					  for(i=0; i<7; i++)
		  		        {
				         largeMessage.msg_25[i] =  canFrame_CAN_1_diag_ptr[i+4];
  	      		        }	
						CANFlags.lrgMsgToFrame =  0x55;
					    largeMessage.numBytes = largeMessage.numBytes + 7;

					 break;

					 case 38:
					  for(i=0; i<7; i++)
		  		        {
				         largeMessage.msg_26[i] =  canFrame_CAN_1_diag_ptr[i+4];
  	      		        }	
						CANFlags.lrgMsgToFrame++;
				 		largeMessage.numBytes = largeMessage.numBytes + 7;

					 break;

					 case 39:
					  for(i=0; i<7; i++)
		  		        {
				         largeMessage.msg_27[i] =  canFrame_CAN_1_diag_ptr[i+4];
  	      		        }	
						CANFlags.lrgMsgToFrame = 0x56;
						largeMessage.numBytes = largeMessage.numBytes + 7;

					  break;

					 case 40:
					  for(i=0; i<7; i++)
		  		        {
				         largeMessage.msg_28[i] =  canFrame_CAN_1_diag_ptr[i+4];
  	      		        }
						CANFlags.lrgMsgToFrame =  0x57;
					    largeMessage.numBytes = largeMessage.numBytes + 7;

					 break;
					
					 case 41:
					  for(i=0; i<7; i++)
		  		        {
				         largeMessage.msg_28[i] =  canFrame_CAN_1_diag_ptr[i+4];
  	      		        }
						CANFlags.lrgMsgToFrame =  0x58;
					    largeMessage.numBytes = largeMessage.numBytes + 7;

					 break;
											
					 default:
					 // need to set an error for data overflow
					 break;

					}
							
				 // V70_data.rke = largeMessage.msg_21[3];
				 
				}
				
				// Flow control TO receiver 726 -> 72E
				// used when motoring the key out and 
				// locking the tailgate
	 			if(canFrame_CAN_1_diag.D0 == 0x30)
				  {
				   Nop();
								
				  /* 	if(V70_data.state == RELEASE)
					 {
					mROP_alarm_0_Off();
				      CAN_frame_21();
			   	      CANFlags.cs_CAN_1CAN;					
	    	          CANsend(_B0);
					 CAN_waitForDiagResponse();
					T0CONbits.TMR0ON = 0;	
					// V70_data.released = 1;
					 }
					  else	
						 {
	      				  CAN_frame_21_lock_tailgate();
			   	          CANFlags.cs_CAN_1CAN;					
	    	              CANsend(_B0);
					      CAN_waitForDiagResponse();
					      T0CONbits.TMR0ON = 0;	
						 }  */
					
				  } 
			
							
		    	// Enter Diagnostic state response
				 if(canFrame_CAN_1_diag.D1 == 0x50)
					{
					 Nop();
					/*
				     V70_data.diag_state_CAN_1 = 0x03;
				
					 if(V70_data.state == RELEASE)
					  {	 
					   CAN_ejectKey();					
			  	       CANFlags.cs_CAN_1CAN = 1;
				       CANsend(_B0); 	
						CAN_waitForDiagResponse();
					  // V70_data.released = 1;		 		
					  }

					 if(V70_data.state == KEYOUT)
					  {	 
					   CAN_frame_21_lock_tailgate();				
			  	       CANFlags.cs_CAN_1CAN = 1;
				       CANsend(_B0);
						CAN_waitForDiagResponse();
					   V70_data.bootlocked = 1;
				      }


					 if(V70_data.state == LOCKDOWN)
					  {
 					   CAN_setVirtualKeyPos();
		  		  	   CANFlags.cs_CAN_1CAN = 1;
	   	  		 	   CANsend(_B0);
						CAN_waitForDiagResponse();		 		
 					  }  
					*/
					}
				//ReturnDataByIdentifier
				 if(canFrame_CAN_1_diag.D1 == 0x62)
				  {
					// Act on the relevant data
					WORD PID = canFrame_CAN_1_diag.D2;
					PID = PID << 8;
					PID = PID + canFrame_CAN_1_diag.D3;

					switch(PID)
						{
						 case 0xC150:  // Door Locks Status
							Nop();
							// Values are all in byte 2 	
							//  V70_data.rr_door = bit 3.5
							//  V70_data.lr_door = bit 3.4
							//  V70_data.rf_door = bit 2.1
							//  V70_data.lf_door = bit 2.0
							//  V70_data.boot =    bit 2.4
							
					/*		V70_data.rr_door = canFrame_CAN_1_diag.D6 & 0x20; 
							V70_data.lr_door = canFrame_CAN_1_diag.D6 & 0x10; 
							V70_data.rf_door = canFrame_CAN_1_diag.D5 & 0x02; 
							V70_data.lf_door = canFrame_CAN_1_diag.D5 & 0x01;
							V70_data.bootlocked = canFrame_CAN_1_diag.D5 & 0x10; 

							V70_data.doorstat_req = 1;

	 						break;
						*/	
							default:
							  break;

						}
					} 

				 if(canFrame_CAN_1_diag.D1 == 0x6F)
				  {
					// Act on the relevant data
					WORD PID = canFrame_CAN_1_diag.D2;
					PID = PID << 8;
					PID = PID + canFrame_CAN_1_diag.D3;
					
					switch(PID)
						{
						case 0x403A:
						  Nop();						
//						  V70_data.state = KEYOUT;
 //						  V70_data.released = 1;	
		 				  Nop();
						  T0CONbits.TMR0ON = 0; 
					 	//CAN_RecvDiagResponse();
	 
						  break; 

						case 0x411F:
						  Nop();						
						  // Lock down state
 //						  V70_data.keyPos = 1;
						  
		 				  Nop();
	 						break; 

						
						  default:
							break;
 						 }

				// Catch multiframe responses to 2F
				if(canFrame_CAN_1_diag.D2 == 0x6F)
				  {
					// Act on the relevant data
					WORD PID = canFrame_CAN_1_diag.D3;
					PID = PID << 8;
					PID = PID + canFrame_CAN_1_diag.D4;
					
					switch(PID)
						{
						case 0xC151:
						  Nop();				
//						  mROP_alarm_1_On();		
//						  // Tailgate locked
 //						  V70_data.bootlocked = 1;
//		 				  Nop();
						   T0CONbits.TMR0ON = 0;		
	 						break; 

						  default:
							break;
 						 }

					}

			}	
       
					 	
	if(largeMessage.numBytes >= largeMessage.rcvBytes)
	 {
	  // The VIN number is stored in the msgFrames 21-23  
      //if(V70_data.VINrequest == 1)
	//	{
	  //	  V70_data.VINrcvd = 1;
		//}

	  largeMessage.rcvBytes = 0;
	 }

  	CANFlags.diag_CAN_1CAN = 0; 		   
}


void process_CAN_1CAN_bus_msg(void)
{

	unsigned char testVal;
	//CANFlags.msgs_proc --;
			   // Now check bus messages

		   if(canFrame_CAN_1.IDENTIFIER == CAN_messages.N_MSG)	// 0x12A
		   {
			Nop();
//		    V70_data.n = (canFrame_CAN_1.D6 << 4) >> 4 ;
//		    V70_data.n = V70_data.n << 8;
//			V70_data.n |= canFrame_CAN_1.D7;
			Nop();
		   }

    	  if(canFrame_CAN_1.IDENTIFIER == CAN_messages.VS_MSG)  // 0x1D1
	 	  {
			Nop();
//			V70_data.vs = canFrame_CAN_1.D6;
//		    V70_data.vs = V70_data.vs << 8;
//			V70_data.vs |= canFrame_CAN_1.D7;
			Nop();
		  }
		
		  if(canFrame_CAN_1.IDENTIFIER == CAN_messages.THR_POS_MSG)  // 0x125
	 	  {
			Nop();
			if(canFrame_CAN_1.D3 !=0)
			  {
//				V70_data.thr_pos = 0;		// Not in idle position
			  } 
		 	  else
				  {
//				   V70_data.thr_pos = 1;    // Idle position
				  }
			Nop();
		  }

		  if(canFrame_CAN_1.IDENTIFIER == CAN_messages.EPB_MSG)  // 0x369
	 	  {
			Nop();
			if(canFrame_CAN_1.D5 == 0x03)
			  {
//				V70_data.epb = 1;		   // EPB applied
			  } 
		 	  else
				  {
//				   V70_data.epb = 0;       // EPB Not applied
				  }
			Nop();
		  }

		  if(canFrame_CAN_1.IDENTIFIER == CAN_messages.PRNDL_MSG)  // 0x83
	 	  {
			Nop();
//			V70_data.gear = canFrame_CAN_1.D6 & 0x60; // bits<7:5>
//			V70_data.gear = V70_data.gear >> 5;

			Nop();
		  }	
		
		  if(canFrame_CAN_1.IDENTIFIER == CAN_messages.GEAR_MSG)  // 0x315
	 	  {
			Nop();
	//		V70_data.gear = canFrame.D1;
			if(canFrame_CAN_1.D1 & 0x07 == 0)
			 {
			  //V70_data.gear = canFrame.D1;       // Indication assumes Not in 'P'
//			   V70_data.gear = 0;
			 }
			  else
				  {
//					V70_data.gear = (canFrame_CAN_1.D1 & 0x07);
				  }
			Nop();
		  }
	
		  if(canFrame_CAN_1.IDENTIFIER == CAN_messages.BOO_SW_08_MSG)  // 0x245
	 	  {
			Nop();
			if(canFrame_CAN_1.D6 !=0xC0)
			  {
//				V70_data.boo_sw = 0;		   // Footbrake not applied
			  } 
		 	  else
				  {
//				   V70_data.boo_sw = 1;       // FootBrake applied
				  }
			Nop();
		  }	
		
 		if(canFrame_CAN_1.IDENTIFIER == CAN_messages.BOO_SW_09_MSG)  // 0x160
	 	  {
			Nop();
			if(canFrame_CAN_1.D7 == 0x06)
			  {
//				V70_data.boo_sw = 1;		   // Footbrake applied
			  } 
		 	  else
				  {
//				   V70_data.boo_sw = 0;       // Footbrake not applied
				  }
			Nop();
		  }	

		  if(canFrame_CAN_1.IDENTIFIER == CAN_messages.CLUTCH_MSG)	// 0x127
		   {
			Nop();
			if(canFrame_CAN_1.D0 == 0xD0)
			  {
//				V70_data.clutch_sw = 1;		// Pedal depressed position
			  } 
		 	  else
				  {
//				   V70_data.clutch_sw = 0;    // Released
				  }
			Nop();
		   }			


		 if(canFrame_CAN_1.IDENTIFIER == CAN_messages.DOORS_09_MSG) // 0x3EE 
	 	  {
			Nop();
//			V70_data.doorStatus = canFrame_CAN_1.D2;
			testVal = ~canFrame_CAN_1.D2;
			// Test for correct response
			
//			if((testVal & 0x01) > 0) // xx11 1110 
//			  V70_data.rf_door = 1;
//			else
//			  V70_data.rf_door = 0;
			

//			if((testVal & 0x02) > 0) // xx11 1101 
//			  V70_data.lf_door = 1;
//			else
//			  V70_data.lf_door = 0;
			

//			if((testVal & 0x04) > 0) // xx11 1011 
//			  V70_data.lr_door = 1;
//			else
//			  V70_data.lr_door = 0;
			

//			if((testVal & 0x08) > 0) // xx11 0111 
//			  V70_data.rr_door = 1;
//			else
//			  V70_data.rr_door = 0;
			Nop();

//			if((testVal & 0x10) > 0) // xx10 1111 
//			  V70_data.tailgate = 1;
//			else
//			  V70_data.tailgate = 0;
//			Nop();

//		if(V70_data.state==LOCKING)
//		  V70_data.doorstat_req = 1;

		}


//		if((unsigned long)canFrame.IDENTIFIER == CAN_messages.POWERMODE_MSG)  // 0x160
//		{
//		 Nop();
//		 V70_data.key_pos = canFrame.D5;	
//		 Nop();
//		}


	/* calculate if we are parked
	*   EPB applied
	*   Vehicle Speed = 0
	*	Transmission not in Gear
	*/


	CANFlags.bus_CAN_1CAN = 0;
}

#endif

/** EOF CAN_ProcessMsgLS.c ***************************************************************/
