/********************************************************************
 FileName:		processUSBpacketCAN.c
 Dependencies:	See INCLUDES section
 Processor:		PIC18, PIC24, and PIC32 USB Microcontrollers
 Hardware:		This demo is natively intended to be used on Microchip USB demo
 				boards supported by the MCHPFSUSB stack.  See release notes for
 				support matrix.  This demo can be modified for use on other hardware
 				platforms.
 Complier:  	Microchip C18 (for PIC18), C30 (for PIC24), C32 (for PIC32)
 Company:		Deben Enterprises Ltd.

/** I N C L U D E S **********************************************************/
#include "simbuck_cfg.h"
#include "HardwareProfile.h"

#if defined(SIMBUCK_BASEBOARD_4550)
 #include <p18cxxx.h>
 #include "system/eeprom/eeprom.h"
#endif


#include "USB/usb.h"
#include "USB/usb_function_generic.h"
#include "processUSBpacketCAN.h"
#include "../../user/watch.h"

#include "CAN2515drv.h"
#include "CAN2515user.h"
#include "processUSBpacketCAN.h"


/** V A R I A B L E S ********************************************************/
#pragma udata
extern unsigned char OUTPacket[64];		//User application buffer for receiving and holding OUT packets sent from the host
extern unsigned char INPacket[64];		//User application buffer for sending IN packets to the host
extern USB_HANDLE USBGenericInHandle;
extern USB_HANDLE USBGenericOutHandle;
extern MCU_REGS MCU_regs;

#if defined(SIMBLOCK_DUAL_CAN)
  extern CAN_FLAGS CANFlags;

  extern unsigned char CAN_1_rcv[14];
  extern unsigned char CAN_2_rcv[14];
  extern unsigned char CAN_3_rcv[14];
  extern unsigned char CAN_4_rcv[14];

  extern unsigned short frame_offset = 0x1C; // 28;
  extern unsigned char CAN_update;  // 0000 0000 no updates
                                    // 0000 0001 rcv 1 update
                                    // 0000 0010 rcv 3 update
                                    // 0000 0100 rcv 2 update
                                    // 0000 1000 rcv 4 update

#endif
  /** D E C L A R A T I O N S **************************************************/
#pragma code


/******************************************************************************
 * Function:        void ProcessUSB(void)
 *
 * User USB message structure
 *
 * PreCondition:    USB communications has been established
 *
 * Input:           A Correctly formatted byte stream
 *                           <REQ><CTRL><RECLEN><DATA><DATA..>
 *                  OUTPacket [0]  [1]    [2]    [3]   [..]
 *
 * Output:          The response bytes to the request
 *                          <REQ><CTRL><RECLEN><ERR><DATA><DATA..>
 *                  INPacket [0]  [1]    [2]    [3]  [4]   [5]
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for user USB message
 * 		    Structure(s).
 *
 *
 * Note:            The user is expected to populate this function with user
 *		    relevant data over the USB connection
 */


char ProcessUSBdualCAN(void)
  {
  #if defined(SIMBLOCK_DUAL_CAN)
   char chipselect;
   char offset;

   // echo back the request
    INPacket[0] = OUTPacket[0];
    INPacket[1] = OUTPacket[1];

    chipselect = OUTPacket[1];
    offset = OUTPacket[3];

    switch(OUTPacket[0])
    {
    case DC_DEVICE_VERSION: // 0x41
    {
      INPacket[4] = 0xAB;

      INPacket[2] = 0x05;
      INPacket[3] = DC_SUCCESS;
    }
    break;

    case DC_READ_2515_ADDR:  // 0x4A
       {
        INPacket[3] = DC_SUCCESS;

       	// get the second byte for the Controller
	// get the third byte for the address
		
	if(chipselect == 1)
        {
	 CANFlags.cs_CAN_1 = 0;     // Active low
        }
        else if(chipselect == 2)
        {
	  CANFlags.cs_CAN_2 = 0;  // Active Low
        }
        else
        {
         INPacket[3] = DC_ERROR_CHIPSELECT;
        }

        if(INPacket[3] == DC_SUCCESS)
        {
	 INPacket[6] = CAN2515_getregister(offset);
        }

        if(chipselect == 1)
        {
	 CANFlags.cs_CAN_1 = 1;
        }
        else if(chipselect == 2)
        {
	  CANFlags.cs_CAN_2 = 1;
        }
        else
        {
         INPacket[3] = DC_ERROR_CHIPSELECT;
        }

        INPacket[4] = chipselect;
        INPacket[5] = offset;

        INPacket[2] = 0x07;
        INPacket[3] == DC_SUCCESS;
       }

    break;

    default:
       INPacket[3] = DC_ERROR_RESPONSE;
       INPacket[2] = 0x04;
    }
    #endif
  }
