/********************************************************************
 FileName:	interrupt.c
 Dependencies:	See INCLUDES section
 Processor:	PIC18 and PIC32 USB Microcontrollers
 Hardware:      Simbuck Baseboards and PIC32 USB Starter kit
 Complier:  	Microchip C18 (for PIC18), XC32 (for PIC32)
 Company:	Deben Enterprises Limited


 THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
 File Description:

 Change History:
  Rev   Description
  ----  -----------------------------------------
  1.0  First release


/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include "simbuck_cfg.h"
#include "../HardwareProfile.h"
#include "system/eeprom/eeprom.h"


#if !defined(SIMBUCK_BASEBOARD_4685)
#include "USB/usb.h"
#include "USB/usb_function_generic.h"
#include "../user/processUSBpacket.h"
#include "user/user_4550.h"
#include "../user/userInits.h"
#endif

#if defined(SIMBUCK_BASEBOARD_4685)
#include "user/user_4685.h"
#include "../user/userInits.h"
#endif

#if defined(SIMBLOCK_DUAL_CAN)
#include "../HardwareProfile - SimblockDualCAN.h"
#include "system/CAN/CAN2515drv.h"
#include "system/CAN/CAN2515user.h"

#endif

/** P R O T O T Y P E S  *****************************************************/


/** V A R I A B L E S ********************************************************/

#pragma idata
char temp;
static int tick_count=0;

static unsigned char uartbuff[32];


#pragma udata
// externs
extern USER_FLAGS USERFlags;
extern unsigned char bytestoread[];
extern int byteCnt;
// create a pointer to the serial transmit buffer
static unsigned char *READBUFFER_ptr = &bytestoread[0];

#if defined(SIMBLOCK_DUAL_CAN)
extern CAN_FLAGS CANFlags;
extern CAN_PERF CANPerf;
#endif


/** D E C L A R A T I O N S **************************************************/
#pragma code


//These are your actual interrupt handling routines.
#pragma interrupt YourHighPriorityISRCode

void YourHighPriorityISRCode() {
    
    //Check which interrupt flag caused the interrupt.
    //Service the interrupt
    //Clear the interrupt flag
    //Etc.

    unsigned char data;
    static int bytescount=0;

#if defined(USB_INTERRUPT)
    USBDeviceTasks();
#endif

      #if defined(SIMBUCK_BASEBOARD_4685)

        #if defined(PROGRAMMABLE_WITH_SERIAL_BOOTLOADER)
        if(PIR1bits.RCIF == 1)
        {
         if(RCSTAbits.FERR && PORTEbits.RE2 == 0)
            {
             // receiving BREAK state, soft reboot back into Bootloader mode.
             Reset();
            }
        #endif


         if(USERFlags.RECVUART==1)
         {
          READBUFFER_ptr = &bytestoread[0];
          bytescount=0;
          USERFlags.RECVUART=0;
         }
         if(RCSTA & 0x06) // check for errors
         {
          RCSTAbits.CREN=0;
          data = RCREG;
          *READBUFFER_ptr = data;
          READBUFFER_ptr++;
          RCSTAbits.CREN = 1;
         }

         else
         {

          data = RCREG;
          *READBUFFER_ptr = data;
          READBUFFER_ptr++;
          bytescount++;

         }

          if(bytescount == bytestoread[2])
          {
            Nop();
            USERFlags.RECVBUFF_FULL=1;
            Nop();

          }
         PIR1bits.RCIF = 0;
         
        }
    #endif

#if defined(SIMBLOCK_DUAL_CAN)
    // We could use RB On change but this has reliability issues
    /*   if(INTCONbits.RBIF)
        {
         temp = PORTB;
                                    
         if(!(PORTB & CAN_1_INT) > 0)                // MCP2515 INT pin is Active Low
          {
           CANFlags.int_CAN_1 = 1;
           CANPerf.msgs_rcvd_can_1++;
           Nop();
          }

         if(!(PORTB & CAN_2_INT) > 0)               // MCP2515 INT pin is Active Low
          {
           CANFlags.int_CAN_2 = 1;
           CANPerf.msgs_rcvd_can_2++;
           Nop();
          }
                  
         if(PORTB & CAN_INT_MASK == CAN_INT_MASK)
          {
           INTCONbits.RBIF = 0;
          }

       }
     */
#endif
}
//This return will be a "retfie fast", since this is in a #pragma interrupt section


#pragma interruptlow YourLowPriorityISRCode

void YourLowPriorityISRCode() {

   
    //Check which interrupt flag caused the interrupt.
    //Service the interrupt
    //Clear the interrupt flag
    //Etc.

    if (INTCONbits.TMR0IF) {
        // Reset the Timer 1ms
        TMR0H = 0xF4;
        TMR0L = 0x8E;

         // Reset the Interrupt flag
        INTCONbits.T0IF = 0;
        
        /** test code to know we are running **/
        if (tick_count == 0) {
            tick_count = 60000U;
        }
        tick_count--;

#if defined(USE_EUSART)
         if (tick_count % 10 == 0) {
            // send 1000ms interval Serial messages
           USERFlags.SENDUART = 1;
        }
#endif
        
#if defined(SIMBLOCK_DUAL_CAN)
        if (tick_count % 10 == 0) {
            // send 10ms interval CAN messages
            CANFlags.snd_tmr_10 = 1;
        }

        if (tick_count % 20 == 0) {
            // send 20ms interval CAN messages
            CANFlags.snd_tmr_20 = 1;
        }

        if (tick_count % 50 == 0) {
            // send 50ms interval CAN messages
            CANFlags.snd_tmr_50 = 1;
        }

        if (tick_count % 100 == 0) {
            // send 100ms interval CAN messages
            CANFlags.snd_tmr_100 = 1;
        }

        if (tick_count % 250 == 0) {
            // send 250ms interval CAN messages
            CANFlags.snd_tmr_250 = 1;
        }

        if (tick_count % 500 == 0) {
            // send 500ms interval CAN messages
            CANFlags.snd_tmr_500 = 1;
            mLED_3_Toggle();

        }

        if (tick_count % 1000 == 0) {
            // send 1000ms interval CAN messages
            CANFlags.snd_tmr_1000 = 1;
        }

#endif



    }



} //This return will be a "retfie", since this is in a #pragma interruptlow section

void InitUserInterrupts(void) {

    /** Set interrupt priorities *********************************************/

    //RCONbits.IPEN	 	 = 0;   // PIC16CXXX Compatibility mode
    RCONbits.IPEN                = 1;   // Enable priority levels on interrupts

    INTCON2bits.TMR0IP           = 0;   // TMR0 Overflow  Low Priority Interrupt
    //INTCON2bits.TMR0IP         = 1;	// TMR0 Overflow  High Priority Interrupt

    INTCON2bits.RBIP             = 0;   //  RB Port change - low priority
    //INTCON2bits.RBIP           = 1;   //  RB Port change - high priority

    //INTCON3bits.INT2IP         = 0;   // INT2 interrupt -  low priority
    //INTCON3bits.INT2IP         = 1;   // INT2 interrupt -  high priority

    //INTCON3bits.INT1IP         = 0;   // INT1 interrupt -  low priority
    //INTCON3bits.INT1IP         = 1;   // INT1 interrupt -  high priority

    //IPR1bits.SPPIP             = 1;	// Streaming Parallel Port Read/Write priority
    //IPR1bits.ADIP              = 1; 	// A/D convertor interrupt priority
    IPR1bits.RCIP                = 1;   // EUSART Receive interrupt priority
    //IPR1bits.TXIP              = 0; 	// EUSART Transmit interrupt priority
    //IPR1bits.SSPIP             = 1; 	// Master Synchrnous Serial Port interrupt priority
    //IPR1bits.CCP1IP            = 1; 	// CCP1 interrupt priority
    //IPR1bits.TMR2IP            = 1;	// TMR2 match interrupt high priority
    //IPR1bits.TMR1IP            = 1;   // TMR1 match interrupt high priority

    //IPR2bits.OSCFIP             = 1;      // enables Oscillator fail interrupt priority
    //IPR2bits.CMIP               = 1;	// enables Comparator Interrupt priority
    //IPR2bits.USBIP              = 1;	// enables USB interrupt priority
    //IPR2bits.EEIP               = 1;      // enables EEPROM/Flash write Operation Interrupt priority
    //IPR2bits.BCLIP              = 1;	// enables Bus Collision Interrupt priority
    //IPR2bits.HLVDIP             = 1;	// enables High/Low voltage detect interrupt priority
    //IPR2bits.TMR3IP             = 1;	// enables TMR3 overflow interrupt priority
    //IPR2bits.CCP2IP             = 1;	// enables CCP2 Interrupt priority

    /** Set interrupt edges *********************************************/
    //INTCON2bits.INTEDG0     = 1;          // Rising edge
    //INTCON2bits.INTEDG1     = 1;          // Rising edge
    //INTCON2bits.INTEDG2     = 1;          // Rising edge


    /** Enable Interrupt(s) **************************************************/
    INTCONbits.TMR0IE         = 1;    // Enables the TMR0 overflow interrupt
    //INTCONbits.INT0IE       = 1;    // Enables INT0
    INTCONbits.RBIE           = 0;    // Enables RB on Change interrupt

    //INTCON3bits.INT2E       = 1;    // Enables INT2
    //INTCON3bits.INT1E       = 1;    // Enables INT1

#if !defined(SIMBUCK_BASEBOARD_4685)
    PIE1bits.SPPIE            = 0;    // enables Streaming Parallel Port interrupt
#endif

#if defined(USED_ADC)
    //PIE1bits.ADIE           = 1;    // enables A/D Convertor interrupt
#endif

    PIE1bits.RCIE             = 1;      // enables EUSART Receive interrupt
    //PIE1bits.TXIE           = 1;	// enables EEUSART Transmit interrupt
    //PIE1bits.SSPIE          = 1;	// enables Master Synchronous Serial Port interrupt
    //PIE1bits.CCP1IE         = 1;	// enables CCP1 interrupt
    //PIE1bits.TMR2IE	      = 1;	// enables TMR2 overflow interrupt
    //PIE1bits.TMR1IE         = 1;	// enables TMR1 overflow interrupt

    //PIE2bits.OSCFIE         = 1;	// enables Oscillator fail interrupt
    //PIE2bits.CMIE           = 1;	// enables Comparator Interrupt
    //PIE2bits.USBIE          = 1;	// enables USB interrupt
    //PIE2bits.EEIE           = 1;	// enables EEPROM/Flash write Operation Interrupt
    //PIE2bits.BCLIE          = 1;	// enables Bus Collision Interrupt
    //PIE2bits.HLVDIE         = 1;	// enables High/Low voltage detect interrupt
    //PIE2bits.TMR3IE         = 1;	// enables TMR3 overflow interrupt
    //PIE2bits.CCP2IE         = 1;	// enables CCP2 Interrupt

    /** Clear Interrupt Flag(s) ***************************************************/
    INTCONbits.TMR0IF         = 0;
    //INTCONbits.INT0IF       = 0;
    INTCONbits.RBIF = 0;

    //INTCON3bits.INT2F       = 0;
    //INTCON3bits.INT1F       = 0;
#if defined(SIMBUCK_BASEBOARD_4550)
    PIR1bits.SSPIF            = 0;
#endif

#if defined(SIMBUCK_BASEBOARD_4685)
    PIR1bits.SSPIF            = 1;
#endif

    PIR1bits.ADIF             = 0;
    PIR1bits.RCIF             = 0;
    PIR1bits.TXIF             = 0;
    //PIR1bits.SSPIF          = 0;
    //PIR1bits.CCP1IF          = 0;
    //PIR1bits.TMR2IF          = 0;
    //PIR1bits.TMR1IF	   = 0;

    //PIR2bits.OSCFIF          = 0;
    //PIR2bits.CMIF            = 0;
    //PIR2bits.USBIF           = 0;
    //PIR2bits.EEIF            = 0;
    //PIR2bits.BCLIF           = 0;
    //PIR2bits.HLVDIF          = 0;
    //PIR2bits.TMR3IF          = 0;
    //PIR2bits.CCP2IF          = 0;

    /** Enable Global Interrupt(s) ********************************************/
    //INTCONbits.GIE	  = 1;		// Enables all high priority interrupts
    //INTCONbits.PEIE	  = 1;		// Enables all low priority peripheral interrupts



    /**  TMR0 interrupt	******************************************************

                    T0CON	= 0x27
                    TMR0ON	= 0
                    T08BIT	= 0		// 16 bit
                    T0CS	= 1		// instruction cycle
                    T0SE	= 0		// low to high increment
                    PSA	= 0		// pre scalar assigned
                    T0PS2	= 1		// 1:256
                    T0PS1	= 1
                    T0PS0	= 1
     */

    T0CONbits.TMR0ON            = 1;  // enables TMR0
    // T0CONbits.TMR0ON  	= 0;  // stops TMR0
    // T0CONbits.T08BIT         = 1;  // enables 8bit timer
    T0CONbits.T08BIT            = 0;  // enables 16bit timer
    // T0CONbits.T0CS           = 1;  // Transition on T0CKI pin
    T0CONbits.T0CS              = 0;  // Internal instruction cycle clock (CLKO)
    // T0CONbits.T0SE    	= 1;  // Increment on high-to-low transition on T0CKI pin
    T0CONbits.T0SE              = 0;  // Increment on low-to-high transition on T0CKI pin
    // T0CONbits.PSA   	        = 1;  // Timer0 prescaler is NOT assigned
    T0CONbits.PSA               = 0;  // Timer0 prescaler is assigned

    /** Set clock/edges ******************************************************/
    INTCON2bits.INTEDG2 = 0;

    // Measured data TMR0H/TMR0L = 0 @ 20Mhz
    // T0CON |= 0x07;		// 1:256 	1398.0 mSec max period
    // T0CON |= 0x06;		// 1:128	699.1 mSec max period
    // T0CON |= 0x05;		// 1:64	  	349.5 mSec max period
    // T0CON |= 0x04;		// 1:32	  	174.8 mSec max period
    // T0CON |= 0x03;		// 1:16	  	 87.4 mSec max period
    // T0CON |= 0x02;		// 1:8	   	 43.7 mSec max period
    // T0CON |= 0x01;		// 1:4	  	 21.9 mSec max period
    // T0CON |= 0x00;		// 1:2	  	 10.9 mSec max period

    T0CONbits.T0PS2 = 0;
    T0CONbits.T0PS1 = 0;
    T0CONbits.T0PS0 = 1;
    /**  TMR0 interrupt	******************************************************/

    // Set up TMR2 interrupt
    /**  TMR2 interrupt	******************************************************

    When:
        TMR2 Prescaler = 1
        TMR2 PostScalar = 1
        PR2 = 255
            Fosc = 47058.83 Hz [Full Rate USB clock]
            Period = 0.000021 seconds || 0.2 microseconds

            T2CON	= 0x07
                    unimplemented	= 0		//
                    T2OUTPS3 	= 0		// Timer 2 output PostScale  ||  0000 -> 1:1
                    T2OUTPS2 	= 1		// Timer 2 output PostScale
                    T2OUTPS1 	= 0		// Timer 2 output PostScale
                    T2OUTPS0 	= 0		// Timer 2 output PostScale  ||  1111 -> 1:16
                    TMR2ON		= 1             // Timer 2 on/off
                    T2CKPS1		= 0		// Timer 2 Prescaler	 || 00 -> 1
                    T2CKPS0		= 0		// Timer 2 Prescaler	 || 01 -> 4
                                                                             || 1x -> 16

            TM2 = 253
            PR2 = 255


    /**  TMR2 interrupt	******************************************************/

    //T2CON = 0x10;
    //TMR2 = 00;
    //PR2 = 0xFF;

}

