/********************************************************************
 FileName:		interrupt_speedbox.c
 Dependencies:	See INCLUDES section
 Processor:		PIC18, PIC24, and PIC32 USB Microcontrollers
 Hardware:		This demo is natively intended to be used on Microchip USB demo
 				boards supported by the MCHPFSUSB stack.  See release notes for
 				support matrix.  This demo can be modified for use on other hardware
 				platforms.
 Complier:  	Microchip C18 (for PIC18), C30 (for PIC24), C32 (for PIC32)
 Company:		Deben Enterprises Ltd.

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include "simbuck_cfg.h"
#include "HardwareProfile.h"
#include "../USB/usb.h"
#include "../USB/usb_function_generic.h"
#include "system/eeprom/eeprom.h"

#include "../user/speedbox/speedboxUSBpacket.h"
#include "../user/speedbox/speedboxuser.h"
#include "../user/speedbox/simstat/simstat.h"

#if defined(SIMBLOCK_DUAL_CAN)
  #include "system/CAN/CAN2515drv.h"
  #include "system/CAN/CAN2515user.h"
#endif

#if defined(USE_ADC)
  #include "../system/adc/simbuckadc.h"
#endif

#if defined(DISPLAY_4DSYSTEMS)
  #include "display/display4d.h"
  #include "system/serial/simbuckserial.h"
  #include "system/serial/simbuckserial.h"
#endif


/** E X T E R N S ************************************************************/
extern SIG_CONFIG SIGconfig;

#if defined(SIMBLOCK_DUAL_CAN)
  extern volatile CAN_FLAGS CANFlags;
  extern volatile CAN_PERF CANPerf;
#endif

#if defined(USE_ADC)
  extern struct ADC_USER ADCUser;
  extern struct ADC_VALUES ADCValues;
  extern struct ADC_VALUES *adc_values_ptr;
#endif


/** V A R I A B L E S ********************************************************/
#pragma udata
static unsigned short transitions = -1;
static unsigned int tmrCnt;
static unsigned int RBCnt;
static unsigned short testcnt;

/** D E C L A R A T I O N S **************************************************/
#pragma code


	//These are your actual interrupt handling routines.
	#pragma interrupt YourHighPriorityISRCode
	void YourHighPriorityISRCode()
	{
          int i;
         

		//Check which interrupt flag caused the interrupt.
		//Service the interrupt
		//Clear the interrupt flag
		//Etc.
        #if defined(USB_INTERRUPT)
	        USBDeviceTasks();
        #endif

            

    if (PIR1bits.TMR2IF == 1) {

        PIR1bits.TMR2IF = 0;
        mEDGE_Toggle();
        

    // the simulation is based on counting the edges
    // for the generation of the simulation signals
    // CPS, CID, Rev/Min

        SIGconfig.edgeCtr++;

        if ((SIGconfig.SIGn_active & 0x02)) {
            SIGconfig.CID_edges++;
            SIGconfig.cycle_edges++;
        }

        //deal with the Baseline [CPS] signal first
        if (SIGconfig.edgeCtr < (SIGconfig.num_Teeth << 1)) {
            if (SIGconfig.edgeCtr > (SIGconfig.num_GapTeeth << 1) - 1)
             {
              if(SIGconfig.CPS_Type == BIDIRECT)
               {
                 // Bi-Directional fixed period 'pulse'
                 if(SIGconfig.edgeCtr & 0x01 == 1)
                  {
                    mCPS_Toggle();

                    // it is a small fixed time pulse
                    for (i = 0; i < 21; i++) {
                        Nop();
                    }
                    mCPS_Toggle();
                  }
               } // Left the 'Gap'

             else
              {
                // Standard variable period 'pulse'
                mCPS_Toggle();
              }
             } // Left the 'Gap'
             
            else
              {
                mCPS_Hi(); // inverted by the Opto
              }

        }
         ////// Mark the revolution/cycle //////
        else
           {
            // create rev count
            mREV_Lo(); // inverted by the Opto
            SIGconfig.edgeCtr = 0;         
          }
       
           if (SIGconfig.edgeCtr == (SIGconfig.num_Teeth << 1) - 1)
            {
              mREV_Lo(); // inverted by the Opto
            }
            else
              {
               mREV_Hi(); // inverted by the Opto
              }


        ////// now deal with the CID signal ///////
        // first when is it active
        if (SIGconfig.SIGn_active != 0x02)
         {
          if (SIGconfig.edgeCtr > (SIGconfig.CID_Start << 1) + SIGconfig.num_GapTeeth)
           {
             SIGconfig.SIGn_active += 0x02;
             SIGconfig.CID_edges = 0;
           }
          }

        // CID signal is active
        if (SIGconfig.SIGn_active > 0)
         {
          if (SIGconfig.CID_edges >= SIGconfig.CID_period << 1)
           {
            // 'Period' edges expired change Pin state
            Nop();
            Nop();
                // use the new method to update the period
                  if(SIGconfig.CPS_Type==MULTILOBE)
                   {
                  
                      mCID_Toggle();
                      mCID_inv = !mCID;

                      transitions++;
                      SIGconfig.CID_period = SIGconfig.CID_teeth[transitions];
                      if((SIGconfig.CID_period == 0) || (transitions >= SIGconfig.CID_transitions_max))
                       {
                        transitions = 0;
                        SIGconfig.CID_period = SIGconfig.CID_teeth[0];
                       }
                     }
              
                    SIGconfig.CID_edges = 0;
                   }


        } //
   
}
    INTCON3bits.INT1IF = 0;
    INTCON3bits.INT2IF = 0;


        if (INTCONbits.TMR0IF) {

        

        tmrCnt++;
        // Reset the Timer 1ms
        TMR0H = 0xF4;
        TMR0L = 0x8E;


        if (tmrCnt % 10 == 0) {

            #if defined(SIMBLOCK_DUAL_CAN)
              CANFlags.snd_tmr_10 = 1;
              CANFlags.snd_CAN_1_BUFF = 1;

            #endif

           #if defined(USE_ADC)
           // Settling time for ADC start up expired
           if(ADCUser.power)
           {
            // Get the next ADC read
            ADCUser.getNext = 1;
           }
            #endif

        }


        if (tmrCnt % 100 == 0) {

             #if defined(SIMBLOCK_DUAL_CAN)
              CANFlags.snd_tmr_100 = 1;
              CANFlags.snd_CAN_2_BUFF = 1;

            #endif

            #if defined(USE_ADC)

                 // wait 100ms for ADC to settle
                 if(!ADCUser.power)
                  {
                   ADCUser.power=1;
                   ADCUser.getNext=1;
                   ADCUser.getCounts=1;
                   Nop();
                  }
            #endif


        }

         if (tmrCnt % 1000 == 0) {
             
             mROP_5_Toggle();
         }

        if (tmrCnt > 65500) {
           tmrCnt = 0;

          

        }

        // Reset the Interrupt flag
        INTCONbits.T0IF = 0;
    }


        #if defined(USE_ADC)
           if(PIR1bits.ADIF == 1)
            {
              // clear interrupt flag
              ADCUser.storeCounts = 1;
              ADCUser.getNext = 0;
              PIR1bits.ADIF = 0;
            }

            

        #endif


 
} //This return will be a "retfie fast", since this is in a #pragma interrupt section


	#pragma interruptlow YourLowPriorityISRCode
	void YourLowPriorityISRCode()
	{


		//Check which interrupt flag caused the interrupt.
		//Service the interrupt
		//Clear the interrupt flag
		//Etc.

  /*  if (INTCONbits.RBIF == 1) {
        RBCnt = PORTB;

        if (RBCnt & 0x01) {
            // process message
            CANFlags.msg_CAN_1;
            // clear the interrupt
            CANFlags.clrInt_CAN_1 = 1;
        }

        if (RBCnt & 0x02) {
            // process message
            CANFlags.msg_CAN_2;
            // clear the interrupt
            CANFlags.clrInt_CAN_2 = 1;
        }
        INTCONbits.RBIF = 0;
    }

 */




	}	//This return will be a "retfie", since this is in a #pragma interruptlow section


void InitSpeedboxInterrupts(void)
{
    // Set up Timers

   /**  TMR0 interrupt	******************************************************

		T0CON	= 0x27
		TMR0ON	= 0
		T08BIT	= 0		// 16 bit
		T0CS	= 1		// instruction cycle
		T0SE	= 0		// low to high increment
		PSA	= 0		// pre scalar assigned
		T0PS2	= 1		// 1:256
		T0PS1	= 1
		T0PS0	= 1
*/

	 T0CONbits.TMR0ON	= 1;		// enables TMR0
	// T0CONbits.TMR0ON	  = 0;		// stops TMR0
	// T0CONbits.T08BIT	  = 1;		// enables 8bit timer
	 T0CONbits.T08BIT       = 0;		// enables 16bit timer
	// T0CONbits.T0CS	  = 1;		// Transition on T0CKI pin
	 T0CONbits.T0CS	    	= 0;		// Internal instruction cycle clock (CLKO)
	// T0CONbits.T0SE	  = 1;		// Increment on high-to-low transition on T0CKI pin
	 T0CONbits.T0SE	     	= 0;		// Increment on low-to-high transition on T0CKI pin
	// T0CONbits.PSA	  = 1;		// Timer0 prescaler is NOT assigned
	 T0CONbits.PSA	     	= 0;		// Timer0 prescaler is assigned


	// Measured data TMR0H/TMR0L = 0 @ 20Mhz
	// T0CON |= 0x07;		// 1:256 	1398.0 mSec max period
	// T0CON |= 0x06;		// 1:128	699.1 mSec max period
	// T0CON |= 0x05;		// 1:64	  	349.5 mSec max period
	// T0CON |= 0x04;		// 1:32	  	174.8 mSec max period
	// T0CON |= 0x03;		// 1:16	  	 87.4 mSec max period
	// T0CON |= 0x02;		// 1:8	   	 43.7 mSec max period
	// T0CON |= 0x01;		// 1:4	  	 21.9 mSec max period
	// T0CON |= 0x00;		// 1:2	  	 10.9 mSec max period

	T0CONbits.T0PS2		= 0;
	T0CONbits.T0PS1		= 0;
	T0CONbits.T0PS0		= 1;

   
    // Set up TMR2 interrupt
    /**  TMR2 interrupt	******************************************************

    When:
            TMR2 Prescaler = 1
        TMR2 PostScalar = 1
            PR2 = 255
            Fosc = 47058.83 Hz [Full Rate USB clock]
            Period = 0.000021 seconds || 0.2 microseconds

            T2CON	= 0x07
                    unimplemented	= 0		//
                    T2OUTPS3 		= 0		// Timer 2 output PostScale  ||  0000 -> 1:1
                    T2OUTPS2 		= 1		// Timer 2 output PostScale
                    T2OUTPS1 		= 0		// Timer 2 output PostScale
                    T2OUTPS0 		= 0		// Timer 2 output PostScale  ||  1111 -> 1:16
                    TMR2ON			= 1	    // Timer 2 on/off
                    T2CKPS1			= 0		// Timer 2 Prescaler		 || 00 -> 1
                    T2CKPS0			= 0		// Timer 2 Prescaler		 || 01 -> 4
                                                                                                                             || 1x -> 16

            TM2 = 253
            PR2 = 255


    /**  TMR2 interrupt	******************************************************/

    T2CON = 0x00;
      T2CONbits.T2CKPS0 = 1;
      T2CONbits.T2CKPS1 = 1;

      T2CONbits.T2OUTPS0 = 1;
      T2CONbits.T2OUTPS1 = 1;
      T2CONbits.T2OUTPS2 = 1;
      T2CONbits.T2OUTPS3 = 1;


    // Set up External interrupt pins

    /**  INT2 interrupt	******************************************************

        Enable interrupt int2 -> IPEN = 0

                 INTCON			= 0xE0
                        GIE/GIEH	= 1
                        PEI/GIEL	= 1
                        TMR0IE		= 1
                        INT0IE		= 0
                        RB1E		= 0
                        TMR0IF	 	= 0
                        INT0IF		= 0
                        RBIF		= 0

                 INTCON2		= 0x00
                        RBPU		= 0
                        INTEDG0		= 0
                        INTEDG1		= 0
                        INTEDG2		= 0
                        not used	= 0
                        TMR0IP	 	= 0
                        not used	= 0
                        RBIP		= 0

                 INTCON3		= 0x10
                        INT2IP		= 0
                        INT1IP		= 0
                        not used	= 0
                        INT2IE		= 1
                        INT1IE		= 0
                        not used 	= 0
                        INT2IF		= 0
                        INT1IF		= 0

                 T0CON			= 0x07
                        TMR0ON		= 0
                        T08BIT		= 0			// 16 bit
                        T0CS		= 0			// instruction cycle
                        T0SE		= 0			// low to high increment
                        PSA			= 0 		// pre scalar assigned
                        T0PS2		= 1			// 1:256
                        T0PS1		= 1
                        T0PS0		= 1


	/** Set interrupt priorities *********************************************/

	//RCONbits.IPEN	 	 = 0;		// PIC16CXXX Compatibility mode
        RCONbits.IPEN 	 	 = 1;		// Enable priority levels on interrupts

	//INTCON2bits.TMR0IP     = 0;		// TMR0 Overflow  Low Priority Interrupt
        INTCON2bits.TMR0IP     = 1;		// TMR0 Overflow  High Priority Interrupt

        INTCON2bits.RBIP        = 0;          //  RB Port change - low priority
        //INTCON2bits.RBIP        = 1;            //  RB Port change - high priority

        //INTCON3bits.INT2IP       = 0;         // INT2 interrupt -  low priority
        INTCON3bits.INT2IP       = 1;         // INT2 interrupt -  high priority

        INTCON3bits.INT1IP       = 0;         // INT1 interrupt -  low priority
        //INTCON3bits.INT1IP       = 1;         // INT1 interrupt -  high priority

        IPR1bits.SPPIP             = 0;	        // Streaming Parallel Port Read/Write priority
        IPR1bits.ADIP              = 1; 	// A/D convertor interrupt priority
        IPR1bits.RCIP              = 0; 	// EUSART Receive interrupt priority
        IPR1bits.TXIP              = 0; 	// EUSART Transmit interrupt priority
        IPR1bits.SSPIP             = 0; 	// Master Synchrnous Serial Port interrupt priority
        IPR1bits.CCP1IP            = 0; 	// CCP1 interrupt priority
        IPR1bits.TMR2IP            = 1;	        // TMR2 match interrupt high priority
        IPR1bits.TMR1IP            = 0;         // TMR1 match interrupt high priority

        IPR2bits.OSCFIP            = 0;         // enables Oscillator fail interrupt priority
        IPR2bits.CMIP              = 0;	        // enables Comparator Interrupt priority
        IPR2bits.USBIP             = 0;	        // enables USB interrupt priority
        IPR2bits.EEIP              = 0;         // enables EEPROM/Flash write Operation Interrupt priority
        IPR2bits.BCLIP             = 0;	        // enables Bus Collision Interrupt priority
        IPR2bits.HLVDIP            = 0;	        // enables High/Low voltage detect interrupt priority
        IPR2bits.TMR3IP            = 0;	        // enables TMR3 overflow interrupt priority
        IPR2bits.CCP2IP            = 0;	        // enables CCP2 Interrupt priority

	/** Set interrupt edges *********************************************/
        INTCON2bits.INTEDG0     = 1;          // Rising edge
        INTCON2bits.INTEDG1     = 1;          // Rising edge
        INTCON2bits.INTEDG2     = 1;          // Rising edge



    /** Clear Interrupt Flag(s) ***************************************************/
	INTCONbits.TMR0IF	 = 0;
        INTCONbits.INT0IF        = 0;
        INTCONbits.RBIF          = 0;

        INTCON3bits.INT2F        = 0;
        INTCON3bits.INT1F        = 0;

#if !defined(SIMBUCK_BASEBOARD_4685)
         PIR1bits.SPPIF           = 0;
        #endif
        PIR1bits.ADIF            = 0;
        PIR1bits.RCIF            = 0;
        PIR1bits.TXIF            = 0;
        PIR1bits.SSPIF           = 0;
        PIR1bits.CCP1IF          = 0;
        PIR1bits.TMR2IF          = 0;
        PIR1bits.TMR1IF	         = 0;

        PIR2bits.OSCFIF          = 0;
        PIR2bits.CMIF            = 0;
        PIR2bits.USBIF           = 0;
        PIR2bits.EEIF            = 0;
        PIR2bits.BCLIF           = 0;
        PIR2bits.HLVDIF          = 0;
        PIR2bits.TMR3IF          = 0;
        PIR2bits.CCP2IF          = 0;


    	/** Enable Interrupt(s) **************************************************/
	INTCONbits.TMR0IE	 = 1;		// Enables the TMR0 overflow interrupt
        INTCONbits.INT0IE        = 0;           // Enables INT0
        INTCONbits.RBIE          = 0;           // Enables RB on Change interrupt

        INTCON3bits.INT2E        = 1;         // Enables INT2
        INTCON3bits.INT1E        = 0;         // Enables INT1

        #if !defined(SIMBUCK_BASEBOARD_4685)
         PIE1bits.SPPIE           = 0;          // enables Streaming Parallel Port interrupt
        #endif
         PIE1bits.ADIE            = 1;	// enables A/D Convertor interrupt
         PIE1bits.RCIE            = 0;  // enables EUSART Receive interrupt
         PIE1bits.TXIE            = 0;	// enables EEUSART Transmit interrupt
         PIE1bits.SSPIE           = 0;	// enables Master Synchronous Serial Port interrupt
         PIE1bits.CCP1IE          = 0;	// enables CCP1 interrupt
         PIE1bits.TMR2IE	  = 1;	// enables TMR2 overflow interrupt
         PIE1bits.TMR1IE          = 0;	// enables TMR1 overflow interrupt

         PIE2bits.OSCFIE          = 0;	// enables Oscillator fail interrupt
         PIE2bits.CMIE            = 0;	// enables Comparator Interrupt
         PIE2bits.USBIE           = 0;	// enables USB interrupt
         PIE2bits.EEIE            = 0;	// enables EEPROM/Flash write Operation Interrupt
         PIE2bits.BCLIE           = 0;	// enables Bus Collision Interrupt
         PIE2bits.HLVDIE          = 0;	// enables High/Low voltage detect interrupt
         PIE2bits.TMR3IE          = 0;	// enables TMR3 overflow interrupt
         PIE2bits.CCP2IE          = 0;	// enables CCP2 Interrupt


    INTCONbits.GIE = 1; // Enables all high priority interrupts
    INTCONbits.PEIE = 1; // Enables all low priority peripheral interrupts
  }