/********************************************************************
 FileName:	processSerialpacket.c
 Dependencies:	See INCLUDES section
 Processor:	PIC18 and PIC32 USB Microcontrollers
 Hardware:      Simbuck Baseboards and PIC32 USB Starter kit
 Complier:  	Microchip C18 (for PIC18), C32 (for PIC32)
 Company:	Deben Enterprises Limited


 THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

 ********************************************************************
 File Description:

 Change History:
  Rev   Description
  ----  -----------------------------------------
 1.0.0 First release for Simbuck -

 ********************************************************************/


/* I N C L U D E S **********************************************************/
#include "simbuck_cfg.h"
#include "HardwareProfile.h"
#include "processUSBpacket.h"
#include "userInits.h"
#include "../system/serial/simbuckserial.h"



#if defined(SIMBUCK_BASEBOARD_4550)
#include <p18cxxx.h>
#include "system/eeprom/eeprom.h"
#include "userInits.h"
#include "watch.h"
#endif


#if defined(SIMBUCK_BASEBOARD_4685)
#include <p18cxxx.h>
#include "system/eeprom/eeprom.h"
#include "userInits.h"
#include "watch.h"
#endif


#if defined(SIMBLOCK_DUAL_CAN)
#include "HardwareProfile - SimblockDualCAN.h"
#include "../system/CAN/CAN2515drv.h"
#include "../system/CAN/CAN2515user.h"
#include "../system/CAN/CAN2515SelfTest.h"
#endif

/* V A R I A B L E S ********************************************************/
#pragma udata
extern USER_FLAGS USERFlags;
extern MCU_REGS MCU_regs;

#if defined(SIMBUCK_BASEBOARD_4550)
extern unsigned char OUTPacket[64];
extern unsigned char bytestosend[64];
#endif



extern unsigned char uartbyte;        ///< data for UART buffer
extern unsigned char bytestosend[];   ///< buffer for UART data Tx
extern unsigned char bytestoread[];   ///< buffer for UART data Rx
extern unsigned char *BUFFER_ptr;     ///< pointer to buffer data


#if defined(SIMBLOCK_DUAL_CAN)
extern CAN_FLAGS CANFlags;
extern CAN_PERF CANPerf;

extern unsigned char CAN_1_rcv[14];
extern unsigned char CAN_2_rcv[14];
extern unsigned char CAN_3_rcv[14];
extern unsigned char CAN_4_rcv[14];
extern unsigned char CAN_update; // 0000 0000 no updates
// 0000 0001 rcv 1 update
// 0000 0010 rcv 3 update
// 0000 0100 rcv 2 update
// 0000 1000 rcv 4 update

unsigned short frame_offset = 0x1C; ///< the length of the receiver buffer in bytes;

#endif


/* D E C L A R A T I O N S **************************************************/
#pragma code

/**
 * User USB message structure
 *
 * PreCondition:    USB communications has been established
 *
 * Input:           A Correctly formatted byte stream
 *                           <REQ><CTRL><RECLEN><DATA><DATA..>
 *                  OUTPacket [0]  [1]    [2]    [3]   [..]
 *
 * Output:          The response bytes to the request
 *                          <REQ><CTRL><RECLEN><ERR><DATA><DATA..>
 *                  bytestosend [0]  [1]    [2]    [3]  [4]   [5]
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for user USB message
 * 		    Structure(s).
 *
 *
 * Note:            The user is expected to populate this function with user
 *		    relevant data over the USB connection
 */

void ProcessSerial(void)
{
#if defined(USE_EUSART)
  char bytescnt = 0;
  char offset = 0;
  char i=0;
  char reclen = 0;

    BUFFER_ptr = &bytestosend[0];
       
    // echo back the request
    bytestosend[0] = bytestoread[0];
    bytestosend[1] = bytestoread[1];
    bytestosend[2] = 0xFF;
    bytestosend[3] = 0xFF;

    bytestosend[7] =  0x00; // default to NO Bootloader

   switch (bytestoread[0]) //Data arrived
    {
        case DEVICE_VERSION: // Version request 0x10
        {
         switch (bytestoread[1]) {
           {
            case 1:

            break;

         
#if defined(SIMBUCK_BASEBOARD_4550)
        case 2: // Baseboard firmware version
  #if defined(HID_BOOTLOADER)
             bytestosend[4] = STACK_YEAR_USB;   // USB Bootloader Stack
             bytestosend[5] = STACK_MONTH_USB;
             bytestosend[6] = STACK_DAY_USB;

             bytestosend[7] = BOOTLOAD_TYPE;

             bytestosend[8] = SIMBUCK_BOOT_MAJOR;   // Simbuck Bootloader
             bytestosend[9] = SIMBUCK_BOOT_MINOR;   // HID bootloader derivative
             bytestosend[10] = SIMBUCK_BOOT_DOT;

             bytescnt += 11;
  #endif
             bytestosend[8] = MAJOR_VERSION_BASE;     // Firmware
             bytestosend[9] = MINOR_VERSION_BASE;
             bytestosend[10] = DEBUG_VERSION_BASE;

             bytescnt +=3;

    #if defined(MY_4550_DEVICE)
             bytestosend[14] = DEBUG_VERSION_DEVICE; // Device firmware
             bytestosend[15] = MINOR_VERSION_DEVICE;
             bytestosend[16] = MAJOR_VERSION_DEVICE;

             bytescnt +=3;
    #endif
#endif

#if defined(SIMBUCK_BASEBOARD_4685)
       case 2: // Baseboard firmware version
 #if defined(SERIAL_BOOTLOADER)

             bytestosend[4] = MAJOR_VERSION_SERIAL;   // Serial Bootloader AN1310 v1.05
             bytestosend[5] = MINOR_VERSION_SERIAL;
             bytestosend[6] = DOT_VERSION_SERIAL;

             bytestosend[7] = BOOTLOAD_TYPE;

             bytestosend[8] = SIMBUCK_BOOT_MAJOR;   // Simbuck Bootloader
             bytestosend[9] = SIMBUCK_BOOT_MINOR;   // AN1310 v1.05 derivative
             bytestosend[10] = SIMBUCK_BOOT_DOT;

             bytescnt += 11;
  #endif

             bytestosend[11] = MAJOR_VERSION_BASE;     // Firmware
             bytestosend[12] = MINOR_VERSION_BASE;
             bytestosend[13] = DEBUG_VERSION_BASE;
             
             bytescnt +=3;

    #if defined(MY_4685_DEVICE)
             bytestosend[14] = DEBUG_VERSION_DEVICE;
             bytestosend[15] = MINOR_VERSION_DEVICE;
             bytestosend[16] = MAJOR_VERSION_DEVICE;

             bytescnt +=3;
    #endif
#endif             
            bytestosend[2] = bytescnt;
            bytestosend[3] = SUCCESS;

             }
            
            break;

           }
         }
         break;
        

     case READ_EEPROM: // 0x14 Read the EEPROM data from PC application.
        { // we use the second byte for the data segment
            offset = bytestoread[1];
            bytestosend[4] = ReadFromEEPROM(offset);

            bytestosend[2] = 0x05;
            bytestosend[3] = SUCCESS;
        }
            break;

        case WRITE_EEPROM: // 0x15 Write the EEPROM data from PC application.
        { // we use the second byte for the data segment
            offset = bytestoread[1];
            reclen = bytestoread[2];

            for (i = 0; i < 6; i++) {
                WriteToEEPROM(bytestoread[2 + i], offset + i);
                bytestosend[4 + i] = bytestoread[2 + i];
            }

            bytestosend[2] = 0x0A;
            bytestosend[3] = SUCCESS;

        }
            break;

        case GET_BLOCK: // 0x16 Read Config record from EEPROM
         {   // we use byte 1 for the data segment
             // we use byte 2 for the size of the message
             // we use byte 3 to define number bytes to read
            offset = bytestoread[1];
            reclen = bytestoread[3];

            for (i = 0; i < reclen; i++) {
                bytestosend[4 + i] = ReadFromEEPROM(offset + i);
            }

            bytestosend[2] = reclen + 3;
            bytestosend[3] = SUCCESS;
        }
            break;


         /* self test (Simbuck-JPF) */
          case GET_FUSES: //0x20
            {
             TBLPTR = (unsigned short long) 0x300000;

             for (i = 0; i < 14; i++) {
             _asm
		tblrdpostinc
 	    _endasm

             bytestosend[i+4] = TABLAT;

             }

             TBLPTR = (unsigned short long) 0x3FFFFE;
              _asm
		tblrdpostinc
             _endasm
              i++;
             bytestosend[i+4] = TABLAT;

              _asm
		tblrdpostinc
             _endasm
              i++;
             bytestosend[i+4] = TABLAT;

             bytestosend[2] = i+4;
             bytestosend[3] = SUCCESS;
            }

         break;


        case GET_USER_STATUS: //0x22 the register setup
         {
            bytestosend[4] = MCU_regs._TRISE;
            bytestosend[5] = MCU_regs._TRISD;
            bytestosend[6] = MCU_regs._TRISC;
            bytestosend[7] = MCU_regs._TRISB;
            bytestosend[8] = MCU_regs._TRISA;
            bytestosend[9] = MCU_regs._PIE1;
            bytestosend[10] = MCU_regs._PIE2;
            bytestosend[11] = MCU_regs._PIR1;
            bytestosend[12] = MCU_regs._PIR2;
            bytestosend[13] = MCU_regs._IPR1;
            bytestosend[14] = MCU_regs._IPR2;
            bytestosend[15] = MCU_regs._RCON;

            bytestosend[16] = MCU_regs._INTCON;
            bytestosend[17] = MCU_regs._INTCON2;
            bytestosend[18] = MCU_regs._INTCON3;

            bytestosend[2] = 19;
            bytestosend[3] = SUCCESS;
        }
            break;

        case GET_PORTS: //0x23
        {
            bytestosend[4] = MCU_regs._PORTE;
            bytestosend[5] = MCU_regs._PORTD;
            bytestosend[6] = MCU_regs._PORTC;
            bytestosend[7] = MCU_regs._PORTB;
            bytestosend[8] = MCU_regs._PORTA;
            bytestosend[9] = MCU_regs._LATE;
            bytestosend[10] = MCU_regs._LATD;
            bytestosend[11] = MCU_regs._LATC;
            bytestosend[12] = MCU_regs._LATB;
            bytestosend[13] = MCU_regs._LATA;

            bytestosend[2] = 14;
            bytestosend[3] = SUCCESS;
        }
            break;

            case READ_PORTS:  // 0x3A
            #if defined(SIMBUCK_BASEBOARD_4550)
                bytestosend[4] = PORTA;
            #endif
                bytestosend[5] = PORTB;
                bytestosend[6] = PORTC;
                bytestosend[7] = PORTD;
                bytestosend[8] = PORTE;

                bytestosend[2] = 0x09;
                bytestosend[3] = SUCCESS;

            break;


             case MONITOR:  // 0x3B
              bytestosend[4] = MCU_regs._PORTE;
              bytestosend[5] = MCU_regs._PORTD;
              bytestosend[6] = MCU_regs._PORTC;
              bytestosend[7] = MCU_regs._PORTB;
              bytestosend[8] = MCU_regs._PORTA;
              bytestosend[9] = MCU_regs._LATE;
              bytestosend[10] = MCU_regs._LATD;
              bytestosend[11] = MCU_regs._LATC;
              bytestosend[12] = MCU_regs._LATB;
              bytestosend[13] = MCU_regs._LATA;

              bytestosend[14] = MCU_regs._TRISE;
              bytestosend[15] = MCU_regs._TRISD;
              bytestosend[16] = MCU_regs._TRISC;
              bytestosend[17] = MCU_regs._TRISB;
              bytestosend[18] = MCU_regs._TRISA;
              bytestosend[19] = MCU_regs._PIE1;
              bytestosend[20] = MCU_regs._PIE2;
              bytestosend[21] = MCU_regs._PIR1;
              bytestosend[22] = MCU_regs._PIR2;
              bytestosend[23] = MCU_regs._IPR1;
              bytestosend[24] = MCU_regs._IPR2;
              bytestosend[25] = MCU_regs._RCON;

              bytestosend[26] = MCU_regs._INTCON;
              bytestosend[27] = MCU_regs._INTCON2;
              bytestosend[28] = MCU_regs._INTCON3;

                bytestosend[2] = 0x29;
                bytestosend[3] = SUCCESS;

             break;


       default:
           if(bytestosend[3] != SUCCESS)
             {
              bytestosend[3] = ERROR_RESPONSE;
              bytestosend[4] = NOT_SUPPORTED;
              bytestosend[2] = 0x05;
             }

     }
 
   // send the response
    WriteBytes(bytestosend[2]);
   // reset the READBUFFER pointer in the interrupt
   USERFlags.RECVUART=1;

#endif
}
