/********************************************************************
 FileName:		userInits.c
 Dependencies:	        See INCLUDES section
 Processor:		PIC18 and PIC32 USB Microcontrollers
 Hardware:		This demo is intended to be used on Simbuck Baseboard4550
                        Baseboard4685 and BaseboardPIC32
 Complier:  	        Microchip C18 (for PIC18), C32 (for PIC32)
 Company:		Deben Enterprises Ltd



 THIS SOFTWARE IS PROVIDED IN AN ?AS IS? CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
 File Description:

 Change History:
  Rev   Description
  ----  -----------------------------------------
  1.0  First release

********************************************************************

/** I N C L U D E S **********************************************************/

#include "simbuck_cfg.h"
#include "../HardwareProfile.h"
#include "GenericTypeDefs.h"
#include "userInits.h"
#include "watch.h"

#if defined(SIMBUCK_BASEBOARD_4550)
  #include <p18cxxx.h>
  #include "user_4550.h"
  #include "../system/interrupt/interrupt.h"
  #include "userInits.h"
#endif 

#if defined(SIMBUCK_BASEBOARD_4685)
  #include <p18cxxx.h>
  #include "user_4685.h"
  #include "../system/interrupt/interrupt.h"
  #include "userInits.h"
  
#endif

#if defined USE_EUSART
  #include "../system/serial/simbuckserial.h"
#endif



#if defined(SIMBLOCK_DUAL_CAN)
  #include "../HardwareProfile - SimblockDualCAN.h"
  #include "../system/spi/simbuckspi.h"
  #include "../system/CAN/CAN2515drv.h"
  #include "../system/CAN/CAN2515user.h"
  #include "../system/CAN/CAN_MsgList.h"
#endif


#if defined(USE_ADC)
  #include "../system/adc/simbuckadc.h"
#endif

/** V A R I A B L E S ********************************************************/
#pragma udata

#if defined(SIMBUCK_BASEBOARD_4550)
extern BOOL blinkStatusValid;
extern USER_FLAGS USERFlags;       ///< structure for user semaphore
#endif

#if defined(SIMBUCK_BASEBOARD_4685)
extern BOOL blinkStatus;
extern USER_FLAGS USERFlags;

#endif

 #if defined(SIMBLOCK_DUAL_CAN)
extern CAN_FLAGS CANFlags;
extern CAN_PERF CANPerf;
#endif

#if defined(USE_EUSART)
 extern int byteCnt;
#endif

#if defined(USE_ADC)
  extern struct ADC_USER ADCUser;
  extern struct ADC_VALUES ADCValues;
  extern struct ADC_VALUES *adc_values_ptr;
#endif

/** D E C L A R A T I O N S ***************************************************/
#pragma code

/** U S E R   I N I T I A L I S A T I O N *************************************/


/**
 * User initialisation of user defined data
 *
 * PreCondition:    All configurations and setup function have been executed
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is the entry point for the users initialisations
 *
 * Note:            None
*/
void UserInit(void)
{
 /** B A S E B O A R D - 4 5 5 0   I N I T I A L I S A T I O N ****************/

      /* Set any default user configuration here */
      #if defined(SIMBUCK_BASEBOARD_4550)
        // debugging switch
        USERFlags.DEBUG = 0;
        USERFlags.slowUpdate = 1;
        USERFlags.fastUpdate = 1;
     
      	ADCON0 = 0x00;			// Turn off A/D
	ADCON1 |= 0x0F;			// used as a digital input/output

	CCP1CONbits.P1M0 = 0;           // Switch off comparators
	CCP1CONbits.P1M1 = 0;

        LATA = 0x00;
        PORTA = 0x00;

        LATB = 0x00;
        PORTB = 0x00;

        LATC = 0x00;
        PORTC = 0x00;

        /** Initialise User Structures **/
        InitStructs();

      /** Initialise SelfTest Structures **/
        InitHostAppStruct();

      /** enable LED on RE2 **/
       TRISEbits.TRISE2 = 1;
       LATEbits.LATE2 = 1;

       mInitAllLEDs();

       // Tutorial 1
       TRISD = 0x00;
       LATD = 0x00;
       PORTD = 0x00;

       /** set up Interrupts **/
       InitUserInterrupts();

       //Blink the USB state on the LEDs.
      #if defined(USE_STATUS_LED)
        blinkStatusValid = TRUE;
      #endif

      #endif

 /** B A S E B O A R D - 4 6 8 5   I N I T I A L I S A T I O N ****************/

      /* Set any default user configuration here */
      #if defined(SIMBUCK_BASEBOARD_4685)
        // debugging switch
        USERFlags.DEBUG = 0;
        USERFlags.slowUpdate = 1;
        USERFlags.fastUpdate = 1;

      	ADCON0 = 0x00;			// Turn off A/D
	ADCON1 |= 0x0F;			// used as a digital input/output

	CCP1CONbits.CCP1M = 0;           // Switch off comparators
	ECCP1CONbits.ECCP1M = 0;


        LATA = 0x00;
        PORTA = 0x00;

        LATB = 0x00;
        PORTB = 0x00;

        LATC = 0x00;
        PORTC = 0x00;

        /** Initialise User Structures **/
        InitStructs();

      /** Initialise SelfTest Structures **/
        InitHostAppStruct();

       /** enable LED on RE2 **/
       TRISEbits.TRISE2 = 1;
       LATEbits.LATE2 = 1;

       mInitAllLEDs();

       // Tutorial 1
       TRISD = 0x00;
       LATD = 0x00;
       PORTD = 0x00;


      // The reason we use this chipset!
      #if defined(USE_ECAN)
        ECAN_init();
        // if we want to send the data via CAN

       // Make CAN Tx registers available in Access Bank
       CANCONbits.WIN2 = 1;
       CANCONbits.WIN1 = 1;
       CANCONbits.WIN0 = 1;

       // load a frame for testing
       //ECANCtrlMessage();
       ECAN_DataFrame_1_Message();
       ECAN_DataFrame_2_Message();
       ECAN_DataFrame_3_Message();
       ECAN_DataFrame_4_Message();
       ECAN_DataFrame_5_Message();
     #endif

       // Used for comms via FTDI CP210x USB-UART bridge
       #if defined USE_EUSART
        InitUART();
        byteCnt=0;
        USERFlags.SENDUART = 1;
       #endif

       /** set up Interrupts **/
       InitUserInterrupts();

       //Blink the state on the LEDs.
      #if defined(USE_STATUS_LED)
        blinkStatus = TRUE;
      #endif

      #endif



/** S I M B L O C K   I N I T I A L I S A T I O N ********************************/


        /** set up SPI comms for the 2515 Controllers**/
      #if defined(SIMBLOCK_DUAL_CAN)
        mInitCANPins();
        // make sure nothing is selected
        DeSelect_CAN_1();
        DeSelect_CAN_2();

        InitSPI();

        // now we set up the CAN controllers
        #if defined(USE_CAN_1)
        Select_CAN_1();
         CANFlags.cs_CAN_1 = 0;
	  CAN2515_setcontroller(0);
         CANFlags.cs_CAN_1 = 1;
        DeSelect_CAN_1();

        // Replace with the actual message(s) configuration call
          Select_CAN_1();
           CANFlags.cs_CAN_1 = 0;
             myCAN1diagmessage();
	   CANFlags.cs_CAN_1 = 1;
          DeSelect_CAN_1();

          // Replace with the actual message(s) configuration call
          Select_CAN_1();
           CANFlags.cs_CAN_1 = 0;
             myCANdiagmessage();
	   CANFlags.cs_CAN_1 = 1;
          DeSelect_CAN_1();
        #endif



        #if defined(USE_CAN_2)
          #if defined(USE_CAN_1)
           Select_CAN_2();
         CANFlags.cs_CAN_2 = 0;
	  CAN2515_setcontroller(1);
         CANFlags.cs_CAN_2 = 1;
        DeSelect_CAN_2();

        // Replace with the actual message(s) configuration call
          Select_CAN_2();
           CANFlags.cs_CAN_2 = 0;
             myCAN2diagmessage();
	   CANFlags.cs_CAN_2 = 1;
          DeSelect_CAN_2();
          #else
            #error CAN_1 MUST be enabled for the CAN_2 Clock input
          #endif
        #endif
      #endif

      #if defined(SIMBLOCK_STEPDOWN_ADC)

       ADC_init();
       // Clear ADC measurement register
       ADRESH = 0;
       ADRESL = 0;

    #endif

       T0CONbits.TMR0ON = 1;

          /** Enable Global Interrupt(s) ********************************************/
	  INTCONbits.GIE	 	 = 1;	// Enables all high priority interrupts
	  INTCONbits.PEIE	 	 = 1;	// Enables all low priority peripheral interrupts

      #if defined(SIMBUCK_BASEBOARD_4550)
        T2CONbits.TMR2ON               = 1;   // Start Timer 2
      #endif

}//end UserInit


/**
 * User initialisation of defined structures
 *
 * PreCondition:    All configurations and setup function have been executed
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is the entry point for the users functions
 *
 * Note:            None
*/
void InitStructs(void)
{
    // debugging switches
    USERFlags.DEBUG = 0;
    USERFlags.fastUpdate = 1;
    USERFlags.slowUpdate = 1;

    #if defined(USE_EUSART)
     USERFlags.SENDUART=0;
     USERFlags.RECVUART=0;
     USERFlags.RECVBUFF_FULL=0;
     USERFlags.SENDBUFF=0;
    #endif

   #if defined(SIMBLOCK_DUAL_CAN)
    
    CANFlags.DEBUG=0;
    CANFlags.ALLMSGS=0;
    CANFlags.cs_CAN_2=1;
    CANFlags.cs_CAN_1=1;
    CANFlags.snd_tmr_10=1;
    CANFlags.snd_tmr_20=0;
    CANFlags.snd_tmr_50=0;
    CANFlags.snd_tmr_100=0;
    CANFlags.snd_tmr_250=0;
    CANFlags.snd_tmr_500=0;
    CANFlags.snd_tmr_1000=0;
    CANFlags.snd_CAN_2_BUFF=0;
    CANFlags.snd_CAN_1_BUFF=0;
    CANFlags.err_CAN_2=0;
    CANFlags.err_CAN_1=0;
    CANFlags.int_CAN_1=0;
    CANFlags.int_CAN_2=0;

    CANPerf.msgs_sent_can_1=0;
    CANPerf.msgs_rcvd_can_1=0;
    CANPerf.msgs_sent_can_2=0;
    CANPerf.msgs_rcvd_can_2=0;
    CANPerf.msgs_missed_can_1=0;
    CANPerf.msgs_missed_can_2=0;
    #endif


#if defined(USE_ADC)
    ADCUser.DEBUG=0;
    ADCUser.power=0;
    ADCUser.mode=0;
    ADCUser.getNext=0;
    ADCUser.getCounts=0;
    ADCUser.storeCounts=0;
    ADCUser.loadTx=0;
    ADCUser.sendTx=0;
    ADCUser.sendTx0=0;
    ADCUser.sendTx1=0;
    ADCUser.sendTx2=0;
    ADCUser.useCAN=0;

    #if defined(USE_ECAN)
     ADCUser.useCAN=1;
    #endif

    ADCUser.useUART=0;
    #if defined(USE_EUSART)
     ADCUser.useUART=1;
    #endif

    ADCUser.updateChannel=0;
    ADCUser.activeChannel=2;
    ADCUser.maxChannels = MAX_CHANNELS;
    ADCUser.adcWait=10;
 # endif

}